dnl --------------------------------------------------------------------
dnl Declarations
dnl --------------------------------------------------------------------

dnl ************************
dnl *** AC_FUNC_SNPRINTF ***
dnl ************************

AC_DEFUN([AC_FUNC_SNPRINTF],
[AC_MSG_CHECKING([for working snprintf])
AC_CACHE_VAL(ac_cv_func_snprintf,
[AC_TRY_RUN([
#include<stdio.h>

int main ()
{
  return (!(3 <= snprintf(NULL, 0, "%d", 100)));
}
],
ac_cv_func_snprintf=yes, ac_cv_func_snprintf=no, ac_cv_func_snprintf=no
)])
if test "$ac_cv_func_snprintf" = yes; then
  AC_MSG_RESULT([yes])
  AC_DEFINE(HAVE_SNPRINTF)
else
  AC_MSG_RESULT([no])
fi
])

dnl *************************
dnl *** AC_SNPRINTF_OFLOW ***
dnl *************************

AC_DEFUN([AC_SNPRINTF_OFLOW],
[AC_MSG_CHECKING([whether snprintf checks n])
AC_CACHE_VAL(ac_cv_snprintf_oflow,
[AC_TRY_RUN([
#include <stdio.h>

int main(void)
{
  char ovbuf[7];
  int i;

  for (i = 0; i < 7; i++)
    ovbuf[i] = 'x';
  snprintf(ovbuf, 4, "foo%s", "bar");
  if (ovbuf[5] != 'x')
    exit(1);
  snprintf(ovbuf, 4, "foo%d", 666);
  if (ovbuf[5] != 'x')
    exit(1);
  exit(0);
}
],
ac_cv_snprintf_oflow=no, ac_cv_snprintf_oflow=yes, ac_cv_snprintf_oflow=yes
)])
if test "$ac_cv_snprintf_oflow" = no; then
  AC_MSG_RESULT([yes])
else
  AC_MSG_RESULT([no])
  AC_DEFINE(HAVE_SNPRINTF_OFLOW)
fi
])

dnl *********************************
dnl *** AC_SNPRINTF_NOTTERMINATED ***
dnl *********************************

AC_DEFUN([AC_SNPRINTF_NOTTERMINATED],
[AC_MSG_CHECKING([whether snprintf correctly terminates long strings])
AC_CACHE_VAL(ac_cv_snprintf_notterminated,
[AC_TRY_RUN([
#include <stdio.h>

int main(void)
{
  char b[7];
  int i;

  b[0] = '\0';
  b[6] = '\0';
  for (i = 1; i < 6; i++)
    b[i] = 'x';
  snprintf(b, 5, "123456789");
  return (b[4] != '\0');
}
],
ac_cv_snprintf_notterminated=no, ac_cv_snprintf_notterminated=yes, ac_cv_snprintf_notterminated=no
)])
if test "$ac_cv_snprintf_notterminated" = no; then
  AC_MSG_RESULT([yes])
else
  AC_MSG_RESULT([no])
  AC_DEFINE(HAVE_SNPRINTF_NOTTERMINATED)
fi
])

dnl *********************************
dnl *** AC_STRNCPY_NOTTERMINATED ***
dnl *********************************

AC_DEFUN([AC_STRNCPY_NOTTERMINATED],
[AC_MSG_CHECKING([whether strncpy correctly terminates long strings])
AC_CACHE_VAL(ac_cv_strncpy_notterminated,
[AC_TRY_RUN([
#include <stdio.h>

int main(void)
{
  char b[7];
  int i;

  b[0] = '\0';
  b[6] = '\0';
  for (i = 1; i < 6; i++)
    b[i] = 'x';
  strncpy(b, "123456789", 4);
  return (b[4] != '\0');
}
],
ac_cv_strncpy_notterminated=no, ac_cv_strncpy_notterminated=yes, ac_cv_strncpy_notterminated=no
)])
if test "$ac_cv_strncpy_notterminated" = no; then
  AC_MSG_RESULT([yes])
else
  AC_MSG_RESULT([no])
  AC_DEFINE(HAVE_STRNCPY_NOTTERMINATED)
fi
])

dnl *********************************
dnl *** AC_STRNCAT_NOTTERMINATED ***
dnl *********************************

AC_DEFUN([AC_STRNCAT_NOTTERMINATED],
[AC_MSG_CHECKING([whether strncat correctly terminates long strings])
AC_CACHE_VAL(ac_cv_strncat_notterminated,
[AC_TRY_RUN([
#include <stdio.h>

int main(void)
{
  char b[7];
  int i;

  b[0] = '\0';
  b[6] = '\0';
  for (i = 1; i < 6; i++)
    b[i] = 'x';
  strncat(b, "123456789", 4);
  return (b[4] != '\0');
}
],
ac_cv_strncat_notterminated=no, ac_cv_strncat_notterminated=yes, ac_cv_strncat_notterminated=no
)])
if test "$ac_cv_strncat_notterminated" = no; then
  AC_MSG_RESULT([yes])
else
  AC_MSG_RESULT([no])
  AC_DEFINE(HAVE_STRNCAT_NOTTERMINATED)
fi
])

dnl ********************
dnl *** AC_PROTOTYPE ***
dnl ********************

AC_DEFUN([AC_PROTOTYPE],[
dnl
dnl Upper case function name
dnl
 pushdef([function],translit([$1], [a-z], [A-Z]))
dnl
dnl Collect tags that will be substituted
dnl
 pushdef([tags],[AC_PROTOTYPE_TAGS(builtin([shift],builtin([shift],builtin([shift],$@))))])
dnl
dnl Wrap in a 1 time loop, when a combination is found break to stop the combinatory exploration
dnl
 for i in 1
 do
   AC_PROTOTYPE_LOOP(AC_PROTOTYPE_REVERSE($1, AC_PROTOTYPE_SUBST($2,tags),AC_PROTOTYPE_SUBST($3,tags),builtin([shift],builtin([shift],builtin([shift],$@)))))
   AC_MSG_ERROR($1 unable to find a working combination)
 done
 popdef([tags])
 popdef([function])
])

dnl
dnl AC_PROTOTYPE_REVERSE(list)
dnl
dnl Reverse the order of the <list>
dnl
AC_DEFUN([AC_PROTOTYPE_REVERSE],[ifelse($#,0,,$#,1,[[$1]],[AC_PROTOTYPE_REVERSE(builtin([shift],$@)),[$1]])])

dnl
dnl AC_PROTOTYPE_SUBST(string, tag)
dnl
dnl Substitute all occurence of <tag> in <string> with <tag>_VAL.
dnl Assumes that tag_VAL is a macro containing the value associated to tag.
dnl
AC_DEFUN([AC_PROTOTYPE_SUBST],[ifelse($2,,[$1],[AC_PROTOTYPE_SUBST(patsubst([$1],[$2],[$2[]_VAL]),builtin([shift],builtin([shift],$@)))])])

dnl
dnl AC_PROTOTYPE_TAGS([tag, values, [tag, values ...]])
dnl
dnl Generate a list of <tag> by skipping <values>.
dnl
AC_DEFUN([AC_PROTOTYPE_TAGS],[ifelse($1,,[],[$1, AC_PROTOTYPE_TAGS(builtin([shift],builtin([shift],$@)))])])

dnl
dnl AC_PROTOTYPE_DEFINES(tags)
dnl
dnl Generate a AC_DEFINE(function_tag, tag_VAL) for each tag in <tags> list
dnl Assumes that function is a macro containing the name of the function in upper case
dnl and that tag_VAL is a macro containing the value associated to tag.
dnl
AC_DEFUN([AC_PROTOTYPE_DEFINES],[ifelse($1,,[],[AC_DEFINE(function[]_$1, $1_VAL) AC_PROTOTYPE_DEFINES(builtin([shift],$@))])])

dnl
dnl AC_PROTOTYPE_STATUS(tags)
dnl
dnl Generates a message suitable for argument to AC_MSG_* macros. For each tag
dnl in the <tags> list the message tag => tag_VAL is generated.
dnl Assumes that tag_VAL is a macro containing the value associated to tag.
dnl
AC_DEFUN([AC_PROTOTYPE_STATUS],[ifelse($1,,[],[$1 => $1_VAL AC_PROTOTYPE_STATUS(builtin([shift],$@))])])

dnl
dnl AC_PROTOTYPE_EACH(tag, values)
dnl
dnl Call AC_PROTOTYPE_LOOP for each values and define the macro tag_VAL to
dnl the current value.
dnl
AC_DEFUN([AC_PROTOTYPE_EACH],[
  ifelse($2,, [
  ], [
    pushdef([$1_VAL], $2)
    AC_PROTOTYPE_LOOP(rest)
    popdef([$1_VAL])
    AC_PROTOTYPE_EACH($1, builtin([shift], builtin([shift], $@)))
  ])
])

dnl
dnl AC_PROTOTYPE_LOOP([tag, values, [tag, values ...]], code, include, function)
dnl
dnl If there is a tag/values pair, call AC_PROTOTYPE_EACH with it.
dnl If there is no tag/values pair left, tries to compile the code and include
dnl using AC_TRY_COMPILE. If it compiles, AC_DEFINE all the tags to their
dnl current value and exit with success.
dnl
AC_DEFUN([AC_PROTOTYPE_LOOP],[
 ifelse(builtin([eval], $# > 3), 1,
   [
     pushdef([rest],[builtin([shift],builtin([shift],$@))])
     AC_PROTOTYPE_EACH($2,$1)
     popdef([rest])
   ], [
     AC_MSG_CHECKING($3 AC_PROTOTYPE_STATUS(tags))
dnl
dnl Activate fatal warnings if possible, gives better guess
dnl
     ac_save_CPPFLAGS="$CPPFLAGS"
     ifelse(AC_LANG,CPLUSPLUS,if test "$GXX" = "yes" ; then CPPFLAGS="$CPPFLAGS -Werror" ; fi)
     ifelse(AC_LANG,C,if test "$GCC" = "yes" ; then CPPFLAGS="$CPPFLAGS -Werror" ; fi)
     AC_TRY_COMPILE($2, $1, [
      CPPFLAGS="$ac_save_CPPFLAGS"
      AC_MSG_RESULT(ok)
      AC_PROTOTYPE_DEFINES(tags)
      break;
     ], [
      CPPFLAGS="$ac_save_CPPFLAGS"
      AC_MSG_RESULT(not ok)
     ])
   ]
 )
])

dnl ***************************
dnl *** AC_PROTOTYPE_ACCEPT ***
dnl ***************************

AC_DEFUN([AC_PROTOTYPE_ACCEPT],[
AC_PROTOTYPE(accept,
 [
  #include <sys/types.h>
#include <sys/socket.h>
],
 [
  int a = 0;
  ARG2 * b = 0;
  ARG3 * c = 0;
  accept(a, b, c);
 ],
 ARG2, [struct sockaddr, void],
 ARG3, [socklen_t, size_t, int, unsigned int, long unsigned int])
])

dnl *******************************
dnl *** AC_PROTOTYPE_SETSOCKOPT ***
dnl *******************************

AC_DEFUN([AC_PROTOTYPE_SETSOCKOPT],[
AC_PROTOTYPE(setsockopt,
 [
  #include <sys/types.h>
#include <sys/socket.h>
],
 [
  int a = 0;
  ARG4 * b = 0;
  setsockopt(a, SOL_SOCKET, SO_REUSEADDR, b, sizeof(a));
 ],
 ARG4, [const void, const char, void, char])
])

dnl *********************
dnl *** ACX_FUNC_FORK ***
dnl *********************

AC_DEFUN([ACX_FUNC_FORK],
[AC_MSG_CHECKING(for a working fork)
AC_CACHE_VAL(acx_cv_func_fork_works,
[AC_REQUIRE([AC_TYPE_PID_T])
AC_REQUIRE([AC_HEADER_SYS_WAIT])
AC_TRY_RUN([#include <sys/types.h>
#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

int main()
{
  int status;
  pid_t child = fork();

  if (child < 0) /* Error */
    return (1);
  else if (child == 0) /* Child */
    return (0);

  /* Parent */
  status = (wait(&status) != child);
  return  (status >= 0) ? 0 : 1;
}
],
acx_cv_func_fork_works=yes, acx_cv_func_fork_works=no, acx_cv_func_fork_works=no
)
])
AC_MSG_RESULT($acx_cv_func_fork_works)
if test $acx_cv_func_fork_works = no; then
  AC_DEFINE(HAVE_NO_FORK)
fi
])

dnl **********************
dnl *** TYPE_SOCKLEN_T ***
dnl **********************

AC_DEFUN([TYPE_SOCKLEN_T],
[AC_CACHE_CHECK([for socklen_t], ac_cv_type_socklen_t,
[
  AC_TRY_COMPILE(
  [#include <sys/types.h>
#include <sys/socket.h>],
  [socklen_t len = 42; return 0;],
  ac_cv_type_socklen_t=yes,
  ac_cv_type_socklen_t=no)
])
  if test $ac_cv_type_socklen_t != yes; then
    AC_DEFINE(socklen_t, int)
  fi
])

dnl *******************
dnl *** TYPE_RLIM_T ***
dnl *******************

AC_DEFUN([TYPE_RLIM_T],
[AC_CACHE_CHECK([for rlim_t], ac_cv_type_rlim_t,
[
  AC_TRY_COMPILE(
  [#include <sys/types.h>
#include <sys/resource.h>],
  [rlim_t len = 42; return 0;],
  ac_cv_type_rlim_t=yes,
  ac_cv_type_rlim_t=no)
])
  if test $ac_cv_type_rlim_t != yes; then
    AC_DEFINE(rlim_t, unsigned int)
  fi
])

dnl **********************
dnl *** AC_PROG_APACHE ***
dnl **********************

AC_DEFUN([AC_PROG_APACHE],
[
AC_ARG_WITH(apache,
[  --with-apache[=FILE]    Absolute filename of apache server [httpd/apache in
      	      	      	  \$PATH:/usr/local/apache/sbin:/var/lib/apache/sbin]],
,
[
  withval=yes
]
)
if test "x$withval" != "xno" ; then
  if test "x$withval" == "xyes" ; then
    result=no
    AC_PATH_PROGS(APACHE, apache httpd apache,,[$PATH:/usr/local/apache/sbin:/var/lib/apache/sbin])
    ac_cv_prog_APACHE=$APACHE
    withval=$ac_cv_prog_APACHE
  else
    result=yes
    AC_MSG_CHECKING([for apache])
  fi
  if test "x$withval" != "x"; then
    if test "$result" == "yes"; then
      AC_MSG_RESULT([$withval])
    fi
    ac_cv_prog_APACHE=$withval
    APACHE=$withval
    AC_SUBST(APACHE)
  else
    if test "$result" == "yes"; then
      AC_MSG_RESULT([not found])
    fi
  fi
else
  AC_MSG_RESULT([no])
fi
])

dnl ************************
dnl *** AC_APACHE_CONFIG ***
dnl ************************

AC_DEFUN([AC_APACHE_CONFIG],
[AC_REQUIRE([AC_PROG_APACHE])
AC_MSG_CHECKING([for apache config])
AC_ARG_WITH(apache-config,
[  --with-apache-config[=FILE]
      	      	      	  Absolute filename of apache httpd.conf
			  Needed only to autodetect httpd-user and cgi-bin
                          [parse httpd -V for HTTPD_ROOT/SERVER_CONFIG_FILE]],
,
[
  withval=yes
]
)
if test "x$withval" != "xno" ; then
  if test "x$withval" == "xyes" ; then
    AC_CACHE_VAL(ac_cv_apache_config,[
      root=`$ac_cv_prog_APACHE -V | grep HTTPD_ROOT | sed 's|.*"\(.*\)"|\1|'`
      conf=`$ac_cv_prog_APACHE -V | grep SERVER_CONFIG_FILE | sed 's|.*"\(.*\)"|\1|'`
      if test "x`echo $conf | sed 's|\(.\).*|\1|'`" == "x/"; then
      	ac_cv_apache_config=$conf
      else
      	ac_cv_apache_config=$root/$conf
      fi
    ])
    withval=$ac_cv_apache_config
  fi
  if test "x$withval" == "x"; then
    AC_MSG_RESULT([not found])
  else
    ac_cv_apache_config=$withval
    AC_MSG_RESULT([$withval])
  fi  
else
  AC_MSG_RESULT([no])
fi
])

dnl *********************
dnl *** AC_HTTPD_USER ***
dnl *********************

AC_DEFUN([AC_HTTPD_USER],
[AC_REQUIRE([AC_APACHE_CONFIG])
AC_MSG_CHECKING([for httpd user])
AC_ARG_WITH(httpd-user,
[  --with-httpd-user=USER  Only USER can launch wrapper
      	      	      	  [parse httpd.conf then passwd httpd|www|nobody]],
,[
  withval=yes
])
userguessed=yes
if test "x$withval" != "xno"; then
  if test "x$withval" == "xyes"; then
    AC_CACHE_VAL(ac_cv_httpd_user,[
      if test "x$ac_cv_apache_config" != "x"; then
      	ac_cv_httpd_user=[`cat $ac_cv_apache_config | grep "^[[:space:]]*User[[:space:]]" | head -1 | sed 's|^[[:space:]]*User[[:space:]]\+\([^[:space:]]*\)|\1|'`]
	userguessed=httpd.conf
      fi
      if test "x$ac_cv_httpd_user" == "x"; then
      	if grep -q httpd /etc/passwd; then
      	  ac_cv_httpd_user=httpd
	  userguessed=/etc/passwd
      	else
      	  if grep -q www /etc/passwd; then
      	    ac_cv_httpd_user=www
	    userguessed=/etc/passwd
      	  else
      	    if grep -q nobody /etc/passwd; then
	      ac_cv_httpd_user=nobody
	      userguessed=/etc/passwd
	    else
	      ac_cv_httpd_user=
	      userguessed=
      	    fi
      	  fi
      	fi
      fi
    ])
    withval=$ac_cv_httpd_user
  fi
  if test "x$withval" != "x"; then
    AC_MSG_RESULT([$userguessed, $withval])
    AC_DEFINE_UNQUOTED(CONF_HTTPD_USER, "$withval")
  else
    AC_MSG_RESULT([not found])
  fi
else
  AC_MSG_RESULT([no])
fi
])

dnl **********************
dnl *** AC_HTTP_CGIDIR ***
dnl **********************

AC_DEFUN([AC_HTTPD_CGIDIR],
[AC_REQUIRE([AC_APACHE_CONFIG])
AC_MSG_CHECKING([for httpd cgi-bin])
AC_ARG_WITH(cgi-dir,
[  --with-cgi-dir=DIR      Where to install wrapper [parse httpd.conf]],
,[
  withval=yes
])
if test "x$withval" != "xno"; then
  if test "x$withval" == "xyes"; then
    AC_CACHE_VAL(ac_cv_httpd_cgidir,[
      if test "x$ac_cv_apache_config" != "x"; then
      	ac_cv_httpd_cgidir=[`cat $ac_cv_apache_config | grep "^[[:space:]]*ScriptAlias[[:space:]]\+/cgi-bin/" | head -1 | sed 's|^[[:space:]]*ScriptAlias[[:space:]]\+[^[:space:]]*[[:space:]]\+"\?\([^"[:space:]]*\)"\?|\1|'`]
      fi
    ])
    withval=$ac_cv_httpd_cgidir
  fi
  if test "x$withval" != "x"; then
    if test "x`echo $withval | sed 's|.*\(.\)|\1|'`" == "x/"; then
      withval=[`echo $withval | sed 's|\(.*\)/|\1|'`]
    fi
    AC_MSG_RESULT([$withval])
    cgidir=$withval
    AC_SUBST(cgidir)
  else
    AC_MSG_RESULT([not found])
  fi
else
  AC_MSG_RESULT([no])
fi
])

dnl *****************************
dnl *** AC_HTTPD_DOCUMENTROOT ***
dnl *****************************

AC_DEFUN([AC_HTTPD_DOCUMENTROOT],
[AC_REQUIRE([AC_APACHE_CONFIG])
withval=yes
if test "x$withval" != "xno"; then
  if test "x$withval" == "xyes"; then
    AC_CACHE_VAL(ac_cv_httpd_documentroot,[
      if test "x$ac_cv_apache_config" != "x"; then
      	ac_cv_httpd_documentroot=[`cat $ac_cv_apache_config | grep "^[[:space:]]*DocumentRoot[[:space:]]" | head -1 | sed 's|^[[:space:]]*DocumentRoot[[:space:]]\+"\?\([^"[:space:]]*\)"\?|\1|'`]
      fi
    ])
    withval=$ac_cv_httpd_documentroot
  fi
  if test "x$withval" != "x"; then
    if test "x`echo $withval | sed 's|.*\(.\)|\1|'`" == "x/"; then
      withval=[`echo $withval | sed 's|\(.*\)/|\1|'`]
    fi
    documentroot=$withval
  fi
fi
])

dnl ************************
dnl *** AC_APACHE_SUEXEC ***
dnl ************************

AC_DEFUN([AC_HTTPD_SUEXEC],
[AC_REQUIRE([AC_PROG_APACHE])
AC_MSG_CHECKING([for apache suexec compatibility])
AC_ARG_WITH(suexec,
[  --with-suexec[=FILE]    Enable suEXEC compatibility, located at FILE [no]
                          (yes=>parse httpd -V for HTTPD_ROOT/SUEXEC_BIN)],
,
[
  withval=no
]
)
if test "x$withval" != "xno" ; then
  if test "x$withval" == "xyes" ; then
    AC_CACHE_VAL(ac_cv_httpd_suexec,[
      root=`$ac_cv_prog_APACHE -V | grep HTTPD_ROOT | sed 's|.*"\(.*\)"|\1|'`
      suexec=`$ac_cv_prog_APACHE -V | grep SUEXEC_BIN | sed 's|.*"\(.*\)"|\1|'`
      if test "x`echo $suexec | sed 's|\(.\).*|\1|'`" == "x/"; then
      	ac_cv_httpd_suexec=$suexec
      else
      	ac_cv_httpd_suexec=$root/$suexec
      fi
    ])
    withval=$ac_cv_httpd_suexec
  fi
  if test "x$withval" == "x"; then
    AC_MSG_RESULT([not found])
  else
    ac_cv_httpd_suexec=$withval
    AC_MSG_RESULT([$withval])
    AC_DEFINE_UNQUOTED(CONF_SUEXEC, "$withval")
    SUEXEC=$ac_cv_httpd_suexec
    AC_SUBST(SUEXEC)
  fi
else
  AC_MSG_RESULT([no])
fi
])

dnl ******************************
dnl *** AC_WRAPPER_EXEC_DOMAIN ***
dnl ******************************

AC_DEFUN([AC_WRAPPER_EXEC_DOMAIN],
[AC_REQUIRE([AC_HTTPD_DOCUMENTROOT])
AC_REQUIRE([AC_HTTPD_CGIDIR])
AC_MSG_CHECKING([for wrapper script domain])
AC_ARG_WITH(script-domain,
[  --with-script-domain[=PATH]
      	      	      	  SCRIPT_FILENAME (not WRAPPER_EXEC!) paths/filenames
			  allowed [@cgidir@:@documentroot@:/home]
			  You should only allow webspace directories],
,
[
  withval=yes
]
)
if test "x$withval" != "xno"; then
  if test "x$withval" == "xyes"; then
    withval=[$cgidir:$documentroot:/home]
  fi
  AC_MSG_RESULT([$withval])
  AC_DEFINE_UNQUOTED(CONF_SCRIPT_DOMAIN, "$withval")
else
  AC_MSG_RESULT([unlimited])
fi
])
