#include"exec.h"

extern int debug;
extern char **environ;

int authdelete(char *const argv[])
{
#ifdef CONF_AUTH_EXEC
  char authfilename[] = { "/tmp/maXXXXXX" };
  int authfd;
  FILE *authfile;

#ifdef CONF_AUTH_EXEC_FD
  char tmp[MAXSTRINGSIZE];
#endif
#endif

  if(debug)
  {
#ifdef CONF_AUTH_EXEC
    fprintf(stdout, "%s\n\n", "Authorization exec");
#else
    fprintf(stdout, "%s\n\n", "Authorization");
#endif
  }

#ifdef CONF_AUTH_EXEC
  authfd = mkstemp(authfilename);
  authfile = fdopen(authfd, "w");

#ifdef CONF_AUTH_EXEC_FD
  if(fcntl(authfd, F_SETFD, (fcntl(authfd, F_GETFD)) & ~FD_CLOEXEC))
  {
    wraperror("fcntl CLOEXEC", strerror(errno));	/* doesn't use logging */
    return (-1);
  }
  unlink(authfilename);
#endif

  if(getvar("$HTTP_AUTHORIZATION") != NULL)
  {
    fprintf(authfile, "%%%s=%s\n", "$HTTP_AUTHORIZATION" + 1,
	    getvar("$HTTP_AUTHORIZATION"));
    unsetvar("$HTTP_AUTHORIZATION");
  }
  if(getvar("$HTTP_PROXY_AUTHORIZATION") != NULL)
  {
    fprintf(authfile, "%%%s=%s\n", "$HTTP_PROXY_AUTHORIZATION" + 1,
	    getvar("$HTTP_PROXY_AUTHORIZATION"));
    unsetvar("$HTTP_PROXY_AUTHORIZATION");
  }

  fflush(authfile);

#ifdef CONF_AUTH_EXEC_FD
  snprintf(tmp, MAXSTRINGSIZE, "%d", authfd);
#ifdef HAVE_SNPRINTF_NOTTERMINATED
  tmp[MAXSTRINGSIZE - 1] = '\0';
#endif
  setvar("$WRAPPER_AUTHFD", tmp);
#else
  fclose(authfile);
  setvar("$WRAPPER_AUTHFILE", authfilename);
#endif

  fflush(stdout);

  execve(argv[0], argv, environ);

  wraperrorlog("auth exec", strerror(errno));
  return (-1);
#else /* CONF_AUTH_EXEC */

  setvar("%HTTP_AUTHORIZATION", getvar("$HTTP_AUTHORIZATION"));
  unsetvar("$HTTP_AUTHORIZATION");
  setvar("%HTTP_PROXY_AUTHORIZATION", getvar("$HTTP_PROXY_AUTHORIZATION"));
  unsetvar("$HTTP_PROXY_AUTHORIZATION");

  return (0);
#endif
}

#ifdef CONF_AUTH_EXEC
int authundelete(void)
{
  FILE *authfile;
  char tmp[MAXSTRINGSIZE];
  char var[MAXSTRINGSIZE];
  char val[MAXSTRINGSIZE];

  if(debug)
  {
    fprintf(stdout, "%s\n\n", "Authorization exec resume");
  }

#ifdef CONF_AUTH_EXEC_FD
  authfile = fdopen(atoi(getvar("$WRAPPER_AUTHFD")), "r");
#else
  authfile = fopen(getvar("$WRAPPER_AUTHFILE"), "r");
#endif
  if(authfile == NULL)
  {
    wraperrorlog("fopen authfile", strerror(errno));
    return (-1);
  }

  fseek(authfile, 0, SEEK_SET);

  fgets(tmp, MAXSTRINGSIZE - 1, authfile);
  while(!feof(authfile))
  {
    var[0] = '\0';
    val[0] = '\0';
    sscanf(tmp, "%[^=]=%[^\n\t]", var, val);	/* should be protected by fgets size */

    setvar(var, val);

    fgets(tmp, MAXSTRINGSIZE - 1, authfile);
  }

  fclose(authfile);

#ifdef CONF_AUTH_EXEC_FD
  unsetvar("$WRAPPER_AUTHFD");
#else
  unlink(getvar("$WRAPPER_AUTHFILE"));
  unsetvar("$WRAPPER_AUTHFILE");
#endif

  return (0);
}
#endif

char **createscriptargv(char *script_name, int argc, char **argv,
			int argshift)
{
  char **newargv;
  int i;

  newargv = (char **)malloc((argc + 1) * sizeof(char *));

  if(newargv == NULL)
  {
    wraperrorlog("new argv", "malloc failed");
    return (NULL);
  }

  newargv[0] = strrchr(script_name, '/');
  if(newargv[0] == NULL)
  {
    newargv[0] = script_name;
  }

  for(i = 1; i <= argc - argshift; i++)
  {
    newargv[i] = argv[i + argshift];
  }

  return (newargv);
}

int setprocessgroup(void)	/* could use setsid but apache is already session leader */
{
#ifdef HAVE_SETPGRP
#ifdef SETPGRP_VOID
  if(setpgrp())
#else
  if(setpgrp(0, 0))
#endif
  {
    wraperrorlog("setpgrp failed", strerror(errno));
    return (-1);
  }
#else
#ifdef HAVE_SETPGID
  if(setpgid(0, 0))
  {
    wraperrorlog("setpgid failed", strerror(errno));
    return (-1);
  }
#endif
#endif

  return (0);
}

int safeexecve(const char *path, char *const argv[], char *const envp[])
{
  if(path == NULL)
  {
    wraperrorlog("safeexecv", "path is NULL");
    return (-1);
  }

  if(argv == NULL)
  {
    wraperrorlog("safeexecv", "argv is NULL");
    return (-1);
  }

  execve(path, argv, envp);	/* shouldn't return */

  wraperrorlog("execve", strerror(errno));
  return (-1);
}
