#include"fs.h"

int checkwrapfile(const char *filename)
{
  struct stat sbuf;

  if(lstat(filename, &sbuf))
  {
    wraperrorlog(filename, "doesn't exists or isn't accessible");	/* disclosure but install */
    return (-1);
  }

  if(!S_ISREG(sbuf.st_mode))
  {
    wraperrorlog(filename, "isn't a regular file");	/* disclosure! */
    return (-1);
  }

  if((sbuf.st_mode & S_ISUID) || (sbuf.st_mode & S_ISGID))
  {
    wraperrorlog(filename, "is setuid or setgid. 'chmod a-s file' to correct");	/* disclosure but install */
    return (-1);
  }

  if((sbuf.st_mode & S_IWGRP) || (sbuf.st_mode & S_IWOTH))
  {
    wraperrorlog(filename, "is writable by group or others. 'chmod go-w file' to correct");	/* disclosure but install */
    return (-1);
  }

  if(sbuf.st_uid != 0)
  {
    wraperrorlog(filename, "doesn't belong to root");	/* disclosure but install */
    return (-1);
  }

  return (0);
}

int checkscriptpath(const char *script_filename)
{
  int i;

  if(script_filename == NULL)
  {
    return (-1);
  }

#ifdef CONF_CHECK_PRINTABLE
  for(i = 0; i < strlen(script_filename); i++)
  {
    if(!isprint(script_filename[i]) || isspace(script_filename[i]))
    {
      return (-1);
    }
  }
#endif

  if(strstr(script_filename, "../") != NULL)
  {
    return (-1);
  }

#ifdef CONF_CHECK_DOUBLESLASH
  if(strstr(script_filename, "//") != NULL)
  {
    return (-1);
  }
#endif

  if(strchr(script_filename, '\\') != NULL)
  {
    return (-1);
  }

  return (0);
}

#ifdef CONF_SCRIPT_DOMAIN
int checkscriptdomain(const char *script_filename)
{
  char *domain = CONF_SCRIPT_DOMAIN;
  char *ptr;
  char *oldptr;
  int retval = -1;

  oldptr = domain;
  ptr = strchr(domain, ':');
  if(ptr == NULL)
  {
    ptr = domain + strlen(domain);
  }
  while(ptr != NULL)
  {
    if(!strncmp(script_filename, oldptr, ptr - oldptr)
       && (script_filename[ptr - oldptr] == '/'
	   || script_filename[ptr - oldptr] == '\0'))
      /* last char should be at worst '\0' */
    {
      retval = 0;
      break;
    }

    oldptr = ptr + 1;

    if(*ptr != '\0')
    {
      ptr = strchr(oldptr, ':');
      if(ptr == NULL)
      {
	ptr = domain + strlen(domain);
      }
    }
    else
    {
      ptr = NULL;
    }
  }

  return (retval);
}
#endif

int checkscriptfile(const char *script_filename, const char *script_name, const char *document_root, const uid_t uid, const gid_t gid, int globalscript)	/* globalscript = should not check owner/suid */
{
  struct stat sbuf;

#ifdef CONF_CHECK_DIR
  char dirname[CONF_MAXPATHLEN];
  char *dirslash;
#endif

  if(script_filename == NULL || script_filename[0] == '\0')
  {
    /*wraperrorlog("checkscript_name",
       "NULL or empty script filename (file not found?)"); */
    wrapnotfound();
    return (-1);
  }

  if(checkscriptpath(script_filename))
  {
    wraperrorlog(script_name,
		 "Bad characters in script path (spaces, \"../\", \"//\", \"\\\", ...)");
    return (-1);
  }

#ifdef CONF_SCRIPT_DOMAIN
  if(checkscriptdomain(script_filename))
  {
    wraperrorlog(script_name, "Script outside exec domain");
    return (-1);
  }
#endif

  /* FILE */
  if(lstat(script_filename, &sbuf))
  {
    wraperrorlog(script_name, "doesn't exists or isn't accessible");
    return (-1);
  }

  if(!S_ISREG(sbuf.st_mode))
  {
    wraperrorlog(script_name, "isn't a regular file");
    return (-1);
  }

  if(!globalscript && ((sbuf.st_mode & S_ISUID) || (sbuf.st_mode & S_ISGID)))
  {
    wraperrorlog(script_name,
		 "is setuid or setgid. 'chmod a-s file' to correct");
    return (-1);
  }

  if(!(sbuf.st_mode & S_IXUSR))
  {
    wraperrorlog
      (script_name, "isn't executable by user. 'chmod u+x file' to correct");
    return (-1);
  }

#ifdef CONF_CHECK_SCRIPT_WRITE
  if((sbuf.st_mode & S_IWGRP) || (sbuf.st_mode & S_IWOTH))
  {
    wraperrorlog
      (script_name,
       "is writable by group or others. 'chmod go-w file' to correct");
    return (-1);
  }
#endif

#ifdef CONF_CHECK_SCRIPT_READ
  if(!globalscript && ((sbuf.st_mode & S_IRGRP) || (sbuf.st_mode & S_IROTH)))
  {
    wraperrorlog
      (script_name,
       "is readable by group or others. 'chmod go-r file' to correct");
    return (-1);
  }
#endif

#ifdef CONF_CHECK_SCRIPT_EXEC
  if(!globalscript && ((sbuf.st_mode & S_IXGRP) || (sbuf.st_mode & S_IXOTH)))
  {
    wraperrorlog
      (script_name,
       "is executable by group or others. 'chmod go-x file' to correct");
    return (-1);
  }
#endif

#ifdef CONF_CHECK_SCRIPT_UID
  if(!globalscript && sbuf.st_uid != uid)
  {
    wraperrorlog(script_name, "doesn't belong to the same uid");
    return (-1);
  }
#endif

#ifdef CONF_CHECK_SCRIPT_GID
  if(!globalscript && sbuf.st_gid != gid)
  {
    wraperrorlog(script_name, "doesn't belong to the same gid");
    return (-1);
  }
#endif

#ifdef CONF_CHECK_DIR
  /* DIRS */
  strncpy(dirname, script_filename, CONF_MAXPATHLEN - 1);
#ifdef HAVE_STRNCPY_NOTTERMINATED
  dirname[CONF_MAXPATHLEN - 1] = '\0';
#endif
  dirslash = strrchr(dirname, '/');
  while(!globalscript && dirslash != NULL)
  {
    *dirslash = '\0';
    if(dirname[0] == '/' && document_root != NULL && strstr(dirname, document_root) == NULL)	/* document root (homedir?) ~= dir.* if absolute */
    {
      break;
    }

    if(lstat(dirname, &sbuf))
    {
      wraperrorlog(dirname, "doesn't exists or isn't accessible");	/* disclosure! */
      return (-1);
    }

    if(!S_ISDIR(sbuf.st_mode))
    {
      wraperrorlog(dirname, "isn't a regular directory");	/* disclosure! */
      return (-1);
    }

#ifdef CONF_CHECK_DIR_WRITE
    if((sbuf.st_mode & S_IWGRP) || (sbuf.st_mode & S_IWOTH))
    {
      wraperrorlog(dirname, "is writable by group or others. 'chmod go-w file' to correct");	/* disclosure! */
      return (-1);
    }
#endif

#ifdef CONF_CHECK_DIR_UID
    if(sbuf.st_uid != uid)
    {
      wraperrorlog(dirname, "doesn't belong to the same uid");	/* disclosure! */
      return (-1);
    }
#endif

#ifdef CONF_CHECK_DIR_GID
    if(sbuf.st_gid != gid)
    {
      wraperrorlog(dirname, "doesn't belong to the same gid");	/* disclosure! */
      return (-1);
    }
#endif

#ifdef CONF_CHECK_DIR_1
    dirslash = NULL;
#else
    dirslash = strrchr(dirname, '/');
#endif
  }
#endif

  return (0);
}

int changedir(const char *script_filename, const char *script_name)
{
  char dirname[CONF_MAXPATHLEN];
  char *dirslash;

  dirslash = strrchr(script_filename, '/');
  if(dirslash == NULL)
  {
    wraperrorlog(script_name, "Can't find script directory");
    return (-1);
  }

  if(dirslash - script_filename < CONF_MAXPATHLEN)
  {
    strncpy(dirname, script_filename, dirslash - script_filename);
#ifdef HAVE_STRNCPY_NOTTERMINATED
    dirname[dirslash - script_filename] = '\0';
#endif
  }

  if(chdir(dirname))
  {
    wraperrorlog(dirname, strerror(errno));
    return (-1);
  }

  return (0);
}
