#include"htaccess.h"

extern int debug;

char *request_method = NULL;

char *authbasicdecode(const char *auth)
{
  static char *enctable =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
  char *from;
  char *to;
  char *fptr;
  char *tptr;
  char *tmp;
  int len;

  if(auth == NULL)
  {
    return (NULL);
  }

  len = strlen(auth);
  from = (char *)malloc(sizeof(char) * (len + (len % 4) + 1));

  strcpy(from, auth);
  while((strlen(from) % 4) != 0)
  {
    strcat(from, "=");
  }
  len += (len % 4);
  to = (char *)malloc(sizeof(char) * (((len + 3) / 4) * 3 + 1));

  to[0] = '\0';
  fptr = from;
  tptr = to;
  while(strlen(fptr) > 0)
  {
    tptr[0] = 0;
    tptr[1] = 0;
    tptr[2] = 0;
    if(fptr[0] == '=')
    {
      break;
    }
    if((tmp = strchr(enctable, fptr[0])) == NULL)
    {
      free(from);
      return (NULL);
    }
    tptr[0] |= (tmp - enctable) << 2;
    if(fptr[1] == '=')
    {
      break;
    }
    if((tmp = strchr(enctable, fptr[1])) == NULL)
    {
      free(from);
      return (NULL);
    }
    tptr[0] |= (((tmp - enctable) & 0x30) >> 4);
    tptr[1] |= (((tmp - enctable) & 0x0F) << 4);
    if(fptr[2] == '=')
    {
      tptr++;
      break;
    }
    if((tmp = strchr(enctable, fptr[2])) == NULL)
    {
      free(from);
      return (NULL);
    }
    tptr[1] |= (((tmp - enctable) & 0x3C) >> 2);
    tptr[2] |= (((tmp - enctable) & 0x03) << 6);
    if(fptr[3] == '=')
    {
      tptr += 2;
      break;
    }
    if((tmp = strchr(enctable, fptr[3])) == NULL)
    {
      free(from);
      return (NULL);
    }
    tptr[2] |= (tmp - enctable) & 0x3F;
    fptr += 4;
    tptr += 3;
  }
  tptr[0] = '\0';
  free(from);
  return (to);
}

int htaccessuseringroup(const char *http_user, const char *group,
			const char *authgroupfile)
{
  struct stat sbuf;
  FILE *fich;
  char groupline[CONF_MAXGROUPSIZE];
  char *colonptr;
  char *userptr;

  if(http_user == NULL || group == NULL || authgroupfile == NULL)
  {
    return (-1);
  }

  if(lstat(authgroupfile, &sbuf))
  {
    return (-1);
  }

  if(!S_ISREG(sbuf.st_mode))
  {
    return (-1);
  }

  fich = fopen(authgroupfile, "r");
  if(fich == NULL)
  {
    return (-1);
  }

  fgets(groupline, CONF_MAXGROUPSIZE - 1, fich);
  while(!feof(fich))
  {
    colonptr = strchr(groupline, ':');

    if(colonptr != NULL && strlen(group) == colonptr - groupline
       && !strncmp(group, groupline, strlen(group)))
    {
      userptr = strstr(colonptr, http_user);

      while(userptr != NULL)
      {
	if((*(userptr - 1) == ' ' || *(userptr - 1) == ':')
	   && (*(userptr + strlen(http_user)) == ' '
	       || *(userptr + strlen(http_user)) == '\0'
	       || *(userptr + strlen(http_user)) == '\015'
	       || *(userptr + strlen(http_user)) == '\012'))
	{
	  fclose(fich);
	  return (0);
	}

	userptr = strstr(userptr + 1, http_user);
      }

    }

    fgets(groupline, CONF_MAXGROUPSIZE - 1, fich);
  }

  fclose(fich);
  return (1);
}

int htaccesscheckuser(const char *http_user, const char *http_password,
		      const char *authuserfile, const char *authtype)
{
  struct stat sbuf;
  FILE *fich;
  char passwdline[CONF_MAXUSERSIZE];
  char *colonptr;
  char *passwdptr;

  if(http_user == NULL || http_password == NULL || authuserfile == NULL
     || authtype == NULL)
  {
    return (-1);
  }

  if(lstat(authuserfile, &sbuf))
  {
    return (-1);
  }

  if(!S_ISREG(sbuf.st_mode))
  {
    return (-1);
  }

  fich = fopen(authuserfile, "r");
  if(fich == NULL)
  {
    return (-1);
  }

  fgets(passwdline, CONF_MAXUSERSIZE - 1, fich);
  while(!feof(fich))
  {
    while(strlen(passwdline) > 0
	  && (passwdline[strlen(passwdline) - 1] == '\015'
	      || passwdline[strlen(passwdline) - 1] == '\012'))
    {
      passwdline[strlen(passwdline) - 1] = '\0';
    }

    colonptr = strchr(passwdline, ':');

    if(colonptr != NULL && strlen(http_user) == colonptr - passwdline
       && !strncmp(http_user, passwdline, strlen(http_user)))
    {
      colonptr++;

      if(!strcasecmp(authtype, "Basic"))
      {
	passwdptr = crypt(http_password, colonptr);
	if(passwdptr != NULL && !strcmp(passwdptr, colonptr))
	{
	  fclose(fich);
	  return (0);
	}
	else
	{
	  fclose(fich);
	  return (1);
	}
      }
    }

    fgets(passwdline, CONF_MAXUSERSIZE - 1, fich);
  }

  fclose(fich);
  return (1);
}

int recursehtaccess(const char *script_filename, const char *document_root,
		    const char *request_method,
		    const char *http_authorization)
{
  int retval = 0;
  char *http_authorization_type = NULL;
  char *http_user = NULL;
  char *http_password = NULL;
  char *authptr;
  const char *constptr;
  char path[CONF_MAXPATHLEN];
  char htaccess[CONF_MAXPATHLEN];
  struct stat sbuf;

  if(document_root == NULL || strlen(document_root) < 1)
  {
    return (-1);
  }

  if(http_authorization != NULL)
  {
    constptr = strchr(http_authorization, ' ');
    if(constptr == NULL)
    {
      constptr = http_authorization + strlen(http_authorization);
    }

    http_authorization_type =
      (char *)malloc((constptr - http_authorization + 1) * sizeof(char));

    if(http_authorization_type == NULL)
    {
      return (-1);
    }

    strncpy(http_authorization_type, http_authorization,
	    constptr - http_authorization);
#ifdef HAVE_STRNCPY_NOTTERMINATED
    http_authorization_type[constptr - http_authorization] = '\0';
#endif

    if(!strcmp(http_authorization_type, "Basic"))
    {
      constptr = strchr(http_authorization, ' ');
      if(constptr != NULL)
      {
	constptr++;
      }

      authptr = authbasicdecode(constptr);
      if(authptr != NULL)
      {
	constptr = strchr(authptr, ':');
	if(constptr == NULL)
	{
	  constptr = authptr + strlen(authptr);
	}

	http_user = (char *)malloc((constptr - authptr + 1) * sizeof(char));

	if(http_user == NULL)
	{
	  return (-1);
	}

	strncpy(http_user, authptr, constptr - authptr);
#ifdef HAVE_STRNCPY_NOTTERMINATED
	http_user[constptr - authptr] = '\0';
#endif

	if(*constptr != '\0')
	{
	  constptr++;

	  http_password =
	    (char *)malloc((strlen(constptr) + 1) * sizeof(char));

	  if(http_password == NULL)
	  {
	    return (-1);
	  }

	  strcpy(http_password, constptr);
	}

	free(authptr);
      }
    }

    if(debug)
    {
      fprintf(stdout, "%s=%s:%s\n\n", "Authorization", http_user,
	      http_password);
      fflush(stdout);
    }
  }

  if(!strncmp(script_filename, document_root, strlen(document_root)))
  {
    constptr = script_filename + strlen(document_root);	/* catch last char for / */
    if(*constptr == '/')
    {
      constptr++;
    }
  }
  else
  {
    constptr = strrchr(script_filename, '/');
    if(constptr == NULL)
    {
      constptr = script_filename;
    }
  }

  path[0] = '\0';
  if(constptr - script_filename > 0
     && constptr - script_filename < CONF_MAXPATHLEN)
  {
    strncpy(path, script_filename, constptr - script_filename);
#ifdef HAVE_STRNCPY_NOTTERMINATED
    path[constptr - script_filename] = '\0';
#endif
  }
  if(strlen(path) > 0)
  {
    if(path[strlen(path) - 1] != '/')
    {
      strcat(path, "/");
    }
  }

  do
  {
    if(chdir(path))
    {
      wraperror(path, strerror(errno));
      retval = -1;
      break;
    }

    snprintf(htaccess, CONF_MAXPATHLEN, "%s%s", path, ".htaccess");
#ifdef CONF_SNPRINTF_NOTTERMINATED
    htaccess[CONF_MAXPATHLEN - 1] = '\0';
#endif

    if(lstat(htaccess, &sbuf) == 0)
    {
      if(S_ISREG(sbuf.st_mode))
      {
	if(debug)
	{
	  fprintf(stdout, "htaccess: %s\n\n", htaccess);
	  fflush(stdout);
	}
	if(parse_htaccess
	   (htaccess, script_filename, document_root, request_method,
	    http_authorization_type, http_user, http_password))
	{
	  retval = -1;
	  break;
	}
      }
    }

    constptr = strchr(script_filename + strlen(path), '/');
    if(constptr != NULL)
    {
      constptr++;
    }
    else
    {
      constptr = script_filename + strlen(script_filename);
    }

    path[0] = '\0';
    if(constptr - script_filename > 0
       && constptr - script_filename < CONF_MAXPATHLEN)
    {
      strncpy(path, script_filename, constptr - script_filename);
#ifdef HAVE_STRNCPY_NOTTERMINATED
      path[constptr - script_filename] = '\0';
#endif
    }
  }
  while(strcmp(script_filename, path) && path[0] != '\0');

#ifdef CONF_YYCLEAN
  htaccess_yyclean();
#endif

  if(http_authorization_type != NULL)
  {
    free(http_authorization_type);
  }
  if(http_user != NULL)
  {
    free(http_user);
  }
  if(http_password != NULL)
  {
    free(http_password);
  }
  return (retval);
}
