#ifndef _HTACCESS_H
#define _HTACCESS_H

/*
  WWW-Authenticate: Basic realm="WallyWorld"
  Authorization: Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ==
  HTTP/1.0 401 Unauthorized

  core
  
    AuthName auth-domain
  
    AuthType < Basic | Digest >
      Apache handles only Basic

    Require entity-name entity entity...
      Require user userid userid ...
      Require group group-name group-name ...
      Require valid-user
      
    <Limit method method ... > ... </Limit>
      GET => HEAD
    
    <LimitExcept method method ... > ... </LimitExcept>

    <Files filename> ... </Files>
    
    <FilesMatch regex> ... </FilesMatch>
    
    PARTIAL: ErrorDocument code page
      page=/localurl OK IF NOT SCRIPT
      	  =http://redirect OK
	  ="Message OK
      $HTTP_* -> $REDIRECT_HTTP_*

  mod_auth
  
    AuthGroupFile filename
      mygroup: bob joe anne

    AuthUserFile filename
      user:cryptpasswd

    IGNORED: AuthAuthoritative < on(default) | off >
    
  mod_access
  
    TODO: Order Deny,Allow
      Deny,Allow
      	the Deny directives are evaluated before the Allow directives. (The initial state is OK.)
      Allow,Deny
      	the Allow directives are evaluated before the Deny directives. (The initial state is FORBIDDEN.)
      Mutual-failure
      	==Allow,Deny
    
    TODO: Allow from host host ...
      all
      (partial) domain-name
      full IP address
      partial IP address
      network/netmask
      network/nnn
      
    TODO: Deny from host host ...
*/

#ifdef HAVE_CONFIG_H
#include"config.h"
#endif

#ifdef HAVE_STDIO_H
#include<stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include<stdlib.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include<sys/stat.h>
#endif

#ifdef HAVE_UNISTD_H
#include<unistd.h>
#endif

#ifdef HAVE_CRYPT_H
#include<crypt.h>
#endif

#include"parse_htaccess.h"
#include"var.h"

/* <0 == error, 0 == ok, >0 == no */
char *authbasicdecode(const char *auth);
int htaccessuseringroup(const char *http_user, const char *group,
			const char *authgroupfile);
int htaccesscheckuser(const char *http_user, const char *http_password,
		      const char *authuserfile, const char *authtype);
int recursehtaccess(const char *script_filename, const char *document_root,
		    const char *request_method,
		    const char *http_authorization);
#endif
