#include"id.h"

int checkwrapuser(void)
{
#ifdef CONF_HTTPD_USER
  struct passwd *httpduser;

  if((httpduser = getpwnam(CONF_HTTPD_USER)) == NULL)
  {
    wraperrorlog(CONF_HTTPD_USER, "Compiled HTTPD_USER not found");
    return (-1);
  }
  if(getuid() != httpduser->pw_uid)
  {
    wraperrorlog(httpduser->pw_name, "User isn't compiled HTTPD_USER");
    return (-1);
  }
#endif

#ifdef CONF_CHECK_SETUID
  if(geteuid() != 0)
  {
    wraperrorlog("Install", "Executable isn't setuid");
    return (-1);
  }
#endif

  return (0);
}

int suexecuser(const int argc, char *const argv[])
{
  struct passwd *user;
  struct group *grp;
  char tmp[MAXSTRINGSIZE];

  if(argc < 4)
  {
    wraperror("suEXEC", "not enough arguments");
    return (-1);
  }

  setvar("%WRAPPER_MODE", "suEXEC");

  if(argv[1][0] == '~')		/* ignore userdir mode, we always chdir to script dir as it should */
  {
    setvar("%WRAPPER_USER", argv[1] + 1);
  }
  else
  {
    setvar("%WRAPPER_USER", argv[1]);
  }

  if((user = getpwnam(argv[1])) != NULL)
  {
    snprintf(tmp, MAXSTRINGSIZE, "%d", user->pw_uid);
#ifdef HAVE_SNPRINTF_NOTTERMINATED
    tmp[MAXSTRINGSIZE - 1] = '\0';
#endif
    setvar("%WRAPPER_UID", tmp);
    snprintf(tmp, MAXSTRINGSIZE, "%d", user->pw_gid);
#ifdef HAVE_SNPRINTF_NOTTERMINATED
    tmp[MAXSTRINGSIZE - 1] = '\0';
#endif
    setvar("%WRAPPER_GID", tmp);
    setvar("%WRAPPER_HOME", user->pw_dir);
    setvar("%WRAPPER_SHELL", user->pw_shell);
  }

  if((grp = getgrnam(argv[2])) != NULL)
  {
    snprintf(tmp, MAXSTRINGSIZE, "%d", grp->gr_gid);
#ifdef HAVE_SNPRINTF_NOTTERMINATED
    tmp[MAXSTRINGSIZE - 1] = '\0';
#endif
    setvar("%WRAPPER_GID", tmp);
  }
  else
  {
    wraperror("checkgroup", "Invalid group");
    return (-1);
  }

  setvar("%WRAPPER_SHIFT", "3");

  return (0);
}

int checkuser(const char *name, const uid_t uid, const gid_t gid,
	      const char *shell)
{
#ifdef CONF_CHECK_SHELL
  FILE *etcshells;
  char shellsline[CONF_MAXSHELLSIZE];
  char shellent[CONF_MAXSHELLSIZE];
#endif

  if(name == NULL)
  {
    wraperrorlog("checkuser", "Invalid user");
    return (-1);
  }

  if(uid == 0)
  {
    wraperrorlog(name, "Wanted uid is root");
    return (-1);
  }

#ifdef CONF_MINIMUM_UID
  if(uid < CONF_MINIMUM_UID)
  {
    wraperrorlog(name, "Wanted uid is below minimum uid");
    return (-1);
  }
#endif

  if(gid == 0)
  {
    wraperrorlog(name, "Wanted gid is root");
    return (-1);
  }

#ifdef CONF_MINIMUM_GID
  if(gid < CONF_MINIMUM_GID)
  {
    wraperrorlog(name, "Wanted gid is below minimum uid");
    return (-1);
  }
#endif


#ifdef CONF_CHECK_SHELL
  if(shell == NULL || shell[0] == '\0')
  {
    wraperrorlog(name, "No shell");
    return (-1);
  }

  etcshells = fopen(CONF_SHELLS, "r");
  if(fopen == NULL)
  {
    wraperrorlog("fopen shells", strerror(errno));
    return (-1);
  }

  fgets(shellsline, CONF_MAXSHELLSIZE - 1, etcshells);
  while(!feof(etcshells))
  {

    shellent[0] = '\0';
    sscanf(shellsline, "%[^# \n\t]", shellent);	/* should be protected by fgets size */
    if(!strcmp(shellent, shell))
    {
      break;
    }

    fgets(shellsline, CONF_MAXSHELLSIZE - 1, etcshells);
  }

  if(strcmp(shellent, shell))
  {
    fclose(etcshells);
    wraperrorlog(name, "Not a valid shell");
    return (-1);
  }

  fclose(etcshells);
#endif

  return (0);
}

#if defined(HAVE_SETPAG)
void setpag(void);
#endif

int partialchangeuser(const char *name, const uid_t uid, const gid_t gid)
{
#if defined(HAVE_SETGROUPS) && defined(CONF_USE_SETGROUPS)
  gid_t grouplist[1];

  grouplist[0] = gid;
  if(setgroups(1, grouplist) == -1)
  {
    wraperrorlog("setgroups() failed", strerror(errno));
    return (-1);
  }
#endif

#if defined(HAVE_INITGROUPS) && defined(CONF_USE_INITGROUPS)
  if(initgroups(name, gid) == -1)
  {
    wraperrorlog("initgroups() failed", strerror(errno));
    return (-1);
  }
#endif

#if defined(HAVE_SETFSGID) && defined(HAVE_SETFSUID)
  setfsgid(gid);
  setfsuid(uid);
#elif defined(HAVE_SETEGID) && defined(HAVE_SETEUID)
  setegid(gid);
  seteuid(uid);
#else
#error "Configure error: Can't change uid temporalily!"
#endif

  return (0);
}

int partialrestoreuser(void)
{
#if defined(HAVE_SETGROUPS) && defined(CONF_USE_SETGROUPS)
  gid_t grouplist[1];
#endif

#if defined(HAVE_SETFSGID) && defined(HAVE_SETFSUID)	/* not necessary but why not... */
  setfsuid(0);
  setfsgid(0);
#elif defined(HAVE_SETEGID) && defined(HAVE_SETEUID)
  seteuid(0);
  setegid(0);
#else
#error "Configure error: Can't restore uid temporalily!"
#endif

#if defined(HAVE_SETGROUPS) && defined(CONF_USE_SETGROUPS)
  grouplist[0] = 0;
  if(setgroups(1, grouplist) == -1)
  {
    wraperrorlog("setgroups() failed", strerror(errno));
    return (-1);
  }
#endif

#if defined(HAVE_INITGROUPS) && defined(CONF_USE_INITGROUPS)
  if(initgroups("root", 0) == -1)
  {
    wraperrorlog("initgroups() failed", strerror(errno));
    return (-1);
  }
#endif

  return (0);
}

int changeuser(const char *name, const uid_t uid, const gid_t gid)
{
#if defined(HAVE_SETGROUPS) && defined(CONF_USE_SETGROUPS)
  gid_t grouplist[1];

  grouplist[0] = gid;
  if(setgroups(1, grouplist) == -1)
  {
    wraperrorlog("setgroups() failed", strerror(errno));
    return (-1);
  }
#endif

#if defined(HAVE_INITGROUPS) && defined(CONF_USE_INITGROUPS)
  if(initgroups(name, gid) == -1)
  {
    wraperrorlog("initgroups() failed", strerror(errno));
    return (-1);
  }
#endif

#if defined(HAVE_SETGID) && defined(HAVE_SETUID)
  setgid(gid);
  setuid(uid);
#elif defined(HAVE_SETRESGID) && defined(HAVE_SETRESUID)
  setresgid(gid, gid, gid);
  setresuid(uid, uid, uid);
#elif defined(HAVE_SETREGID) && defined(HAVE_SETREUID)
  setregid(gid, gid);
  setreuid(uid, uid);
#elif defined(HAVE_SETEGID) && defined(HAVE_SETEUID) && defined(HAVE_SETRGID) && defined(HAVE_SETRUID)
  setegid(gid);
  setrgid(gid);
  seteuid(uid);
  setruid(uid);
#else
#error "Configure error: Can't change uid!"
#endif

  if(getuid() != uid)
  {
    wraperrorlog("getuid", "changeuser failed");
    return (-1);
  }
  if(getgid() != gid)
  {
    wraperrorlog("getgid", "changeuser failed");
    return (-1);
  }
  if(geteuid() != uid)
  {
    wraperrorlog("geteuid", "changeuser failed");
    return (-1);
  }
  if(getegid() != gid)
  {
    wraperrorlog("getegid", "changeuser failed");
    return (-1);
  }

  /* check for linux<2.2.16 sendmail-type bug */
  setuid(0);
  if(getuid() == 0)
  {
    wraperrorlog("setuid bug", "Can switch back to uid 0");
    return (-1);
  }

#if defined(HAVE_SETPAG) && defined(CONF_USE_AFS_SETPAG)
  if(setpag())
  {
    wraperrorlog("setpag", strerror(errno));
    return (-1);
  }
#endif

  return (0);
}
