#include"modiwrap.h"

#define RUSAGEMSGSIZE 1024

int nphmodiwrap = 0;
int debug = 0;

#ifdef CONF_SUEXEC
int suEXEC = 0;
#endif

pid_t childpid = 0;

int main(int argc, char **argv)	/* using environ instead of env... strange differences */
{
  char *prgname = NULL;
  char **newenv;
  int childstatus;
  struct rusage childrusage;
  char rusagemsg[RUSAGEMSGSIZE];
  struct timeval stime;
  struct timeval etime;

  if(argv[0] != NULL)
  {
    prgname = strrchr(argv[0], '/');
    if(prgname != NULL)
    {
      prgname++;
    }
    else
    {
      prgname = argv[0];
    }
  }

  if(prgname != NULL && !strncmp(prgname, "nph-", 4))
  {
    nphmodiwrap = 1;
  }

  if(prgname != NULL && strlen(prgname) >= 6
     && !strcmp(prgname + strlen(prgname) - 6, "-debug"))
  {
    debug = 1;
  }

#ifdef CONF_SUEXEC
  if(prgname != NULL && !strcmp(prgname, "suexec"))
  {
    suEXEC = 1;
  }
#endif

#if defined(HAVE_UMASK) && defined(CONF_UMASK)
  umask(CONF_UMASK);
#endif

  if(initlog())
  {
    return (-1);
  }

  if(checkwrapuser())
  {
    return (-1);
  }

  if(debug)
  {
    wraperror("debug", "");
    fprintf(stdout, "start:\n");
    printvars(environ);
    fflush(stdout);
  }

  /* if SECURITY_HOLE_PASS_AUTHORIZATION, move password var to local env and exec to prevent 'ps e' */
  if(getvar("$HTTP_AUTHORIZATION") != NULL
     || getvar("$HTTP_PROXY_AUTHORIZATION") != NULL)
  {
    if(authdelete(argv))
    {
      return (-1);
    }
  }
#ifdef CONF_AUTH_EXEC
  else if(
#ifdef CONF_AUTH_EXEC_FD
	   getvar("$WRAPPER_AUTHFD") != NULL
#else
	   getvar("$WRAPPER_AUTHFILE") != NULL
#endif
    )				/* Detect if we have execve'd */
  {
    if(authundelete())
    {
      return (-1);
    }
  }
#endif


#ifdef CONF_SUEXEC
  if(suEXEC)
  {
    if(suexecuser(argc, argv))
    {
      return (-1);
    }
  }
#endif


#if defined(CONF_MODIWRAP_LOG) && defined(CONF_CHECK_LOG)
  if(checkwrapfile(CONF_MODIWRAP_LOG))	/* could have been created by initlog() */
  {
    return (-1);
  }
#endif

#if defined(CONF_SHELLS) && defined(CONF_CHECK_SHELL)
  if(checkwrapfile(CONF_SHELLS))
  {
    return (-1);
  }
#endif

#ifdef CONF_REDIR_STDERR
  if(dup2(1, 2) < 0)
  {
    wrapwarning("dup2", strerror(errno));
  }
#endif

  /* Config */
  if(checkwrapfile(CONF_MODIWRAP_CONFIG))
  {
    return (-1);
  }

  if(parse_config(CONF_MODIWRAP_CONFIG))
  {
    return (-1);
  }
  /* Config done */

  if(!debug)
  {
    debug = safeatoi(getvar("%WRAPPER_DEBUG"));

    if(debug)
    {
      wraperror("debug", "");
    }
  }

  if(debug)
  {
    fprintf(stdout, "after config:\n");
    printvars(environ);
    fflush(stdout);
  }

  if(getvar("%WRAPPER_REDIRECT") != NULL)
  {
    wrapredirect(getvar("%WRAPPER_REDIRECT"));
    return (0);
  }

  /*if (checkneededvars())
     {
     return (-1);
     } */

  if(checkuser
     (getvar("%WRAPPER_USER"), safeatoi(getvar("%WRAPPER_UID")),
      safeatoi(getvar("%WRAPPER_GID")), getvar("%WRAPPER_SHELL")))
  {
    return (-1);
  }

  partialchangeuser(getvar("%WRAPPER_USER"), safeatoi(getvar("%WRAPPER_UID")), safeatoi(getvar("%WRAPPER_GID")));	/* do file system checks with wrapper userid */

  if(checkscriptfile
     (getvar("$SCRIPT_FILENAME"), getvar("$SCRIPT_NAME"),
      getvar("$DOCUMENT_ROOT"), safeatoi(getvar("%WRAPPER_UID")),
      safeatoi(getvar("%WRAPPER_GID")),
      safeatoi(getvar("%WRAPPER_GLOBALSCRIPT"))))
  {
    return (-1);
  }

  if(changedir(getvar("$SCRIPT_FILENAME"), getvar("$SCRIPT_NAME")))
  {
    return (-1);
  }

#ifdef CONF_USE_HTACCESS
#ifdef CONF_SUEXEC
  if(!suEXEC)			/* suEXEC is already identified by apache if AllowOverride AuthConfig/Limit */
  {
#endif
    if(recursehtaccess
       (getvar("$SCRIPT_FILENAME"), getvar("$DOCUMENT_ROOT"),
	getvar("$REQUEST_METHOD"), getvar("%HTTP_AUTHORIZATION")))
    {
      return (-1);
    }
#ifdef CONF_SUEXEC
  }
#endif
#endif

  partialrestoreuser();		/* needed for rlimits (if raised) & to prevent signal killing */

#ifdef CONF_USE_LIMITS
  if(setrlimitsandalarm
     (getvar("%WRAPPER_USER"), safeatoi(getvar("%WRAPPER_UID")),
      safeatoi(getvar("%WRAPPER_GID")), wrappersignals))
  {
    return (-1);
  }
#endif

#ifdef CONF_FORWARD_SIGNALS
  setwrappersignals();
#endif

  if(gettimeofday(&stime, NULL))
  {
    wraperrorlog("gettimeofday", strerror(errno));
    return (-1);
  }

  fflush(stdout);
  childpid = fork();
  if(childpid < 0)
  {
    if(errno == EAGAIN)
    {
      wraperrorlog("fork failed", "Too many user processes");
    }
    else
    {
      wraperrorlog("fork failed", strerror(errno));
    }
    return (-1);
  }
  else if(childpid == 0)	/* child */
  {
    if(changeuser
       (getvar("%WRAPPER_USER"), safeatoi(getvar("%WRAPPER_UID")),
	safeatoi(getvar("%WRAPPER_GID"))))
    {
      return (-1);
    }

    if(setprocessgroup())
    {
      return (-1);
    }

#ifdef CONF_PARANOID_ENV
    newenv = createsaveenv(environ);
    if(newenv == NULL)
    {
      return (-1);
    }
#else
    newenv = environ;
#endif

    if(debug)
    {
      fprintf(stdout, "before safeexecve:\n");
      printvars(newenv);
      fflush(stdout);
    }

    if(safeexecve
       (getvar("%WRAPPER_EXEC"),
	createscriptargv(getvar("$SCRIPT_FILENAME"), argc, argv,
			 safeatoi(getvar("%WRAPPER_SHIFT"))), newenv))
    {
      return (-1);
    }
  }
  else				/* father */
  {
    cleanlocalvars();

    wait4(childpid, &childstatus, 0, &childrusage);
    killpg(childpid, SIGKILL);	/* kill anything left, just in case :) */
    if(gettimeofday(&etime, NULL))
    {
      wraperrorlog("gettimeofday", strerror(errno));
      return (-1);
    }

    snprintf(rusagemsg, RUSAGEMSGSIZE,
	     "child=%d sig=%d etime=%ld.%06ld utime=%ld.%06ld stime=%ld.%06ld",
	     childpid,
	     ((WIFSIGNALED(childstatus)) ? WTERMSIG(childstatus) :
	      0),
	     (etime.tv_sec - stime.tv_sec) -
	     (((etime.tv_usec - stime.tv_usec) < 0) ? 1 : 0),
	     (etime.tv_usec - stime.tv_usec) +
	     (((etime.tv_usec - stime.tv_usec) < 0) ? 1000000 : 0),
	     childrusage.ru_utime.tv_sec,
	     childrusage.ru_utime.tv_usec,
	     childrusage.ru_stime.tv_sec, childrusage.ru_stime.tv_usec);
#ifdef CONF_SNPRINTF_NOTTERMINATED
    rusagemsg[RUSAGEMSGSIZE - 1] = '\0';
#endif

#if 0				/* not implemented in getrusage() */
    if(childrusage.ru_maxrss != 0)
    {
      snprintf(rusagemsg + strlen(rusagemsg),
	       RUSAGEMSGSIZE - strlen(rusagemsg), " maxrss=%ld",
	       childrusage.ru_maxrss);
#ifdef CONF_SNPRINTF_NOTTERMINATED
      rusagemsg[RUSAGEMSGSIZE - 1] = '\0';
#endif
    }
    if(childrusage.ru_ixrss != 0)
    {
      snprintf(rusagemsg + strlen(rusagemsg),
	       RUSAGEMSGSIZE - strlen(rusagemsg), " ixrss=%ld",
	       childrusage.ru_ixrss);
#ifdef CONF_SNPRINTF_NOTTERMINATED
      rusagemsg[RUSAGEMSGSIZE - 1] = '\0';
#endif
    }
    if(childrusage.ru_idrss != 0)
    {
      snprintf(rusagemsg + strlen(rusagemsg),
	       RUSAGEMSGSIZE - strlen(rusagemsg), " idrss=%ld",
	       childrusage.ru_idrss);
#ifdef CONF_SNPRINTF_NOTTERMINATED
      rusagemsg[RUSAGEMSGSIZE - 1] = '\0';
#endif
    }
    if(childrusage.ru_isrss != 0)
    {
      snprintf(rusagemsg + strlen(rusagemsg),
	       RUSAGEMSGSIZE - strlen(rusagemsg), " isrss=%ld",
	       childrusage.ru_isrss);
#ifdef CONF_SNPRINTF_NOTTERMINATED
      rusagemsg[RUSAGEMSGSIZE - 1] = '\0';
#endif
    }
#endif

    wrapinfo("Exit", rusagemsg);

#ifdef CONF_FORWARD_SIGNALS
    setdefaultwrappersignals();
#endif

    closelog();

    if(WIFSIGNALED(childstatus))
    {
      kill(getpid(), WTERMSIG(childstatus));
    }
    exit(WEXITSTATUS(childstatus));
  }

  return (0);
}
