#define MODIWRAPD
#include"config.h"

#include<stdio.h>
#include<stdlib.h>
#include<errno.h>
#include<string.h>
#include<unistd.h>
#include<sys/stat.h>
#include<sys/wait.h>

#include"rlimitsp.h"

/*
    modiwrapd.limits format:
    
    login: limits...
    # comments
    
    special login '*' == default limits
    limits = modified limits
    -1 == unlimited
    
    ex:
    *: NICE=0 TIME=60 CPU=10 VMEM=2500000 FSIZE=2500000 NFILE=32 NPROC=32
    foo: VMEM=5000000

    if multiple '*' lines, only last is kept
    if multiple lines for same login, behavior undefined (although last should/would be used)

    see available keywords in rlimitsp.h
*/

struct daemonlimitslist
{
  char *dl_login;
  char *dl_limits;
  struct daemonlimitslist *dl_next;
};

#define CRCMAX 256

char *mydefaultlimits;
struct daemonlimitslist *mylimits[CRCMAX];

pid_t reloadpid = 0;
time_t lastmtime;

RETSIGTYPE mysigalrmtimeout(int unused)
{
  exit(-1);
}

RETSIGTYPE mysigusr1(int unused)
{
  exit(0);
}

RETSIGTYPE mysigusr2(int unused)
{
  lastmtime = 0;
}

RETSIGTYPE mysigchld(int unused)
{
  if(wait(NULL) == reloadpid)	/* oops, reload child exited == failed! */
  {
    reloadpid = 0;
  }
}

unsigned char stringcrc8(char *string)	/* don't know if == CRC-8 but seems good one */
{
  int i, j;
  unsigned char check = 0;
  unsigned char b;
  int flag;

  for(i = 0; i < strlen(string); i++)
  {
    b = string[i];
    for(j = 0; j < 8; j++)
    {
      flag = check & 0x80;

      check <<= 1;
      if(b & 0x80)
      {
	check |= 1;
      }
      b <<= 1;
      if(flag)
      {
	check ^= 0x07;		/* crc-8 mask */
      }
    }
  }
  return check;
}

int makelistensocket(void)
{
#ifdef CONF_USE_GETHOSTBYNAME
  struct hostent *he;
#endif
  int listenfd;
  struct sockaddr_in sin;
  int i;

  sin.sin_family = AF_INET;

#ifdef CONF_MODIWRAPD_BIND_ANY

  sin.sin_addr.s_addr = INADDR_ANY;

#else

  if(isipaddr(CONF_LIMITS_DAEMON_ADDR))
  {
    inet_aton(CONF_LIMITS_DAEMON_ADDR, &sin.sin_addr);
  }
  else
  {
#ifdef CONF_USE_GETHOSTBYNAME
    he = gethostbyname(CONF_LIMITS_DAEMON_ADDR);
    if(he == NULL)
    {
      herror("gethostbyname");
      return (-1);
    }
    memcpy(&sin.sin_addr, he->h_addr, he->h_length);
#else
    fprintf(stderr,
	    "gethostbyname support isn't compiled and daemon addr isn't an IP.\n");
    return (-1);
#endif
  }

#endif

  sin.sin_port = htons(CONF_LIMITS_DAEMON_PORT);

  listenfd = socket(AF_INET, SOCK_STREAM, PF_UNSPEC);
  if(listenfd < 0)
  {
    perror("socket");
    return (-1);
  }

  i = -1;
  setsockopt(listenfd, SOL_TCP, SO_KEEPALIVE, (SETSOCKOPT_ARG4*)&i, sizeof(i));
  i = -1;
  setsockopt(listenfd, SOL_TCP, SO_REUSEADDR, (SETSOCKOPT_ARG4*)&i, sizeof(i));

  if(bind(listenfd, (struct sockaddr *)&sin, sizeof(sin)))
  {
    perror("bind");
    return (-1);
  }

  if(listen(listenfd, SOMAXCONN))
  {
    perror("listen");
    return (-1);
  }

  return (listenfd);
}

int loadconf(void)
{
  int i;
  FILE *fich;
  char tmp[MAXSTRINGSIZE];
  char *ptr;
  char *ptr2;
  struct daemonlimitslist *dltmpnext;

  mydefaultlimits = NULL;

  for(i = 0; i < CRCMAX; i++)
  {
    mylimits[i] = NULL;
  }

  fich = fopen(CONF_MODIWRAPD_LIMITS, "r");
  if(fich == NULL)
  {
    return (-1);
  }

  fgets(tmp, MAXSTRINGSIZE, fich);
  while(!feof(fich))
  {
    if(tmp[0] != '\0' && tmp[0] != '#')	/* not a comment */
    {
      ptr = strchr(tmp, ':');
      if(ptr != NULL)
      {
	*ptr = '\0';
	ptr++;
	while(*ptr != '\0' && (*ptr == ' ' || *ptr == '\t'))
	{
	  ptr++;
	}

	ptr2 = ptr + strlen(ptr) - 1;
	while(ptr2 >= ptr && (*ptr2 == '\015' || *ptr2 == '\012'))	/* chomp */
	{
	  *ptr2 = '\0';
	  ptr2--;
	}

	if(!strcmp(tmp, "*"))
	{
	  if(mydefaultlimits != NULL)
	  {
	    free(mydefaultlimits);
	  }

	  mydefaultlimits = (char *)malloc((strlen(ptr) + 1) * sizeof(char));

	  if(mydefaultlimits == NULL)
	  {
	    fclose(fich);
	    return (-1);
	  }

	  strcpy(mydefaultlimits, ptr);
	}
	else
	{
	  i = stringcrc8(tmp);

	  dltmpnext = mylimits[i];

	  mylimits[i] =
	    (struct daemonlimitslist
	     *)malloc(sizeof(struct daemonlimitslist));

	  if(mylimits[i] == NULL)
	  {
	    fclose(fich);
	    return (-1);
	  }

	  mylimits[i]->dl_login =
	    (char *)malloc((strlen(tmp) + 1) * sizeof(char));

	  if(mylimits[i]->dl_login == NULL)
	  {
	    fclose(fich);
	    return (-1);
	  }
	  strcpy(mylimits[i]->dl_login, tmp);

	  mylimits[i]->dl_limits =
	    (char *)malloc((strlen(ptr) + 1) * sizeof(char));

	  if(mylimits[i]->dl_limits == NULL)
	  {
	    fclose(fich);
	    return (-1);
	  }
	  strcpy(mylimits[i]->dl_limits, ptr);

	  mylimits[i]->dl_next = dltmpnext;
	}
      }
    }

    fgets(tmp, MAXSTRINGSIZE, fich);
  }

  fclose(fich);

  return (0);
}

int reloadconf(void)
{
  int i;
  struct daemonlimitslist *dltmp;
  struct daemonlimitslist *dltmpnext;

  if(mydefaultlimits != NULL)
  {
    free(mydefaultlimits);
  }
  /* mydefaultlimits = NULL; */

  for(i = 0; i < CRCMAX; i++)
  {
    dltmp = mylimits[i];

    while(dltmp != NULL)
    {
      dltmpnext = dltmp->dl_next;

      free(dltmp->dl_login);
      free(dltmp->dl_limits);

      free(dltmp);

      dltmp = dltmpnext;
    }

    /*mylimits[i] = NULL; */
  }

  return (loadconf());
}

int getuserlimits(char *user_name, char *buffer, size_t buffersize)
{
  int i;
  struct daemonlimitslist *dltmp;

  if(buffer == NULL || buffersize <= 0)
  {
    return (-1);
  }

  buffer[0] = '\0';
  if(mydefaultlimits != NULL)
  {
    strncpy(buffer, mydefaultlimits, buffersize - 1);
#ifdef HAVE_STRNCPY_NOTTERMINATED
    buffer[buffersize - 1] = '\0';
#endif
  }

  i = stringcrc8(user_name);

  dltmp = mylimits[i];
  while(dltmp != NULL)
  {
    if(!strcmp(dltmp->dl_login, user_name) && buffersize - strlen(buffer) > 1)	/* found */
    {
      strcat(buffer, " ");
      strncpy(buffer + strlen(buffer), dltmp->dl_limits,
	      buffersize - strlen(buffer) - 1);
#ifdef HAVE_STRNCPY_NOTTERMINATED
      buffer[buffersize - 1] = '\0';
#endif
      return (0);
    }
  }

  return (0);
}

int handleincoming(int fd, struct sockaddr_in sin)
{
  uid_t user_uid;
  gid_t user_gid;
  char user_name[CONF_SOCKBUFSIZE];
  char sockbuf[CONF_SOCKBUFSIZE + 3];
  int n;

  signal(SIGALRM, mysigalrmtimeout);	/* request timeout */
  alarm(CONF_MODIWRAPD_REQUEST_TIMEOUT);

  if((n = recv(fd, sockbuf, CONF_SOCKBUFSIZE, 0)) < 0)
  {
    perror("recv");
    return (-1);
  }

  sockbuf[n] = '\0';
  if(sscanf(sockbuf, "%d %d %[^\015\012]", &user_uid, &user_gid, user_name) != 3)	/* bad request! */
  {
    /*perror("sscanf"); */
    return (-1);
  }

  if(getuserlimits(user_name, sockbuf, CONF_SOCKBUFSIZE))
  {
    return (-1);
  }

  strcat(sockbuf, "\015\012");
  if(send(fd, sockbuf, strlen(sockbuf), 0) < 0)
  {
    perror("send");
    return (-1);
  }

  alarm(0);

  return (0);
}

int checkconf(int acceptfd)
{
  struct stat sbuf;

  if(reloadpid == 0 && !lstat(CONF_MODIWRAPD_LIMITS, &sbuf))
  {
    if(sbuf.st_mtime != lastmtime)
    {
      lastmtime = sbuf.st_mtime;

      reloadpid = fork();
      if(reloadpid < 0)
      {
	perror("fork");
	return (-1);
      }
      else if(reloadpid == 0)	/* child */
      {
	if(acceptfd >= 0)
	{
	  close(acceptfd);
	}

	if(reloadconf())	/* failed? */
	{
	  return (-1);
	}

	kill(getppid(), SIGUSR1);

	return (1);
      }
    }
  }

  return (0);
}

RETSIGTYPE mysigalrmconf(int unused)
{
  checkconf(-1);
}

int main(int argc, char **argv)
{
  int listenfd, acceptfd;
  struct stat sbuf;
  struct sockaddr_in sin;
  struct linger lin;
  pid_t pid;
  int i;

  close(0);
  close(1);

  signal(SIGCHLD, mysigchld);
  signal(SIGPIPE, SIG_IGN);

  listenfd = makelistensocket();
  if(listenfd < 0)
  {
    return (-1);
  }

  if(lstat(CONF_MODIWRAPD_LIMITS, &sbuf))
  {
    perror("limits stat");
    return (-1);
  }

  lastmtime = sbuf.st_mtime;
  if(loadconf())		/* failed? */
  {
    return (-1);
  }

  signal(SIGALRM, mysigalrmconf);	/* check conf timeout */

  signal(SIGUSR1, mysigusr1);	/* child completed reload, exit */
  signal(SIGUSR2, mysigusr2);	/* force reload */

  /*fprintf(stderr, "backgrounding...\n"); */
  close(2);
  pid = fork();
  if(pid > 0)
  {
    exit(0);
  }

  while(1)
  {
    i = sizeof(sin);

    alarm(CONF_MODIWRAPD_FORCE_CHECK_CONF_DELAY);
    acceptfd = accept(listenfd, (ACCEPT_ARG2*)&sin, (ACCEPT_ARG3*)&i);
    alarm(0);

    if(acceptfd < 0 && errno != EINTR)
    {
      perror("accept");
    }

    switch (checkconf(acceptfd))
    {
    case -1:			/* error */
      return (-1);
    case 1:			/* reloaded */
      continue;
    default:			/* father or not reloading, nothing to do */
    }

    if(acceptfd >= 0)
    {
#if defined(CONF_MODIWRAPD_ACCEPT_NETADDR) && defined(CONF_MODIWRAPD_ACCEPT_NETMASK)
      if((sin.sin_addr.s_addr & inet_addr(CONF_MODIWRAPD_ACCEPT_NETMASK)) != inet_addr(CONF_MODIWRAPD_ACCEPT_NETADDR))	/* access denied */
      {
	lin.l_onoff = 1;
	lin.l_linger = 0;
	setsockopt(acceptfd, SOL_TCP, SO_LINGER, (SETSOCKOPT_ARG4*)&lin, sizeof(lin));	/* reset connection */

	close(acceptfd);
	continue;
      }
#endif

      pid = fork();
      if(pid < 0)
      {
	perror("fork");
	return (-1);
      }
      else if(pid == 0)		/* child */
      {
	close(listenfd);

	handleincoming(acceptfd, sin);
	close(acceptfd);

	return (0);
      }
      else			/* father */
      {
	close(acceptfd);
      }
    }
  }
}
