#include"output.h"

extern int nphmodiwrap;
extern int debug;

#ifdef CONF_USE_ERRORDOCUMENT
char *error401 = NULL;
char *error404 = NULL;
char *error500 = NULL;
#endif

int logfd = -1;

char loginfo[CONF_MAXLOGINFO] = { "" };

char *syslogctime(void)
{
  char *ptr;
  char *ptr2;
  time_t mytime;

  mytime = time(NULL);
  ptr = ctime(&mytime);

  ptr2 = strrchr(ptr, ' ');	/* before year */
  if(ptr2 != NULL)
  {
    *ptr2 = '\0';
  }

  ptr2 = strchr(ptr, ' ');	/* after week day */
  if(ptr2 != NULL)
  {
    return (ptr2 + 1);
  }

  return (ptr);
}


int initlog(void)
{
  char *remotehost;
  char *remoteport;
  char *serverhost;
  char *serverport;
  char *request;

  remotehost = getvar("$REMOTE_HOST");
  if(remotehost == NULL)
  {
    remotehost = getvar("$REMOTE_ADDR");
  }
  if(remotehost == NULL)
  {
    remotehost = "?";
  }

  remoteport = getvar("$REMOTE_PORT");
  if(remoteport == NULL)
  {
    remoteport = "?";
  }

  serverhost = getvar("$SERVER_NAME");
  if(serverhost == NULL)
  {
    serverhost = getvar("$SERVER_ADDR");
  }
  if(serverhost == NULL)
  {
    serverhost = getvar("$HTTP_HOST");
  }
  if(serverhost == NULL)
  {
    serverhost = "?";
  }

  serverport = getvar("$SERVER_PORT");
  if(serverport == NULL)
  {
    serverport = "?";
  }

  request = getvar("$REQUEST_URI");
  if(request == NULL)
  {
    request = "?";
  }

  snprintf(loginfo, CONF_MAXLOGINFO, "%s:%s %s:%s %s", remotehost, remoteport,
	   serverhost, serverport, request);
#ifdef CONF_SNPRINTF_NOTTERMINATED
  loginfo[CONF_MAXLOGINFO - 1] = '\0';
#endif

#ifdef CONF_USE_SYSLOG
  openlog("modiwrap", LOG_PID | LOG_NOWAIT, LOG_DAEMON);
#endif

#ifdef CONF_MODIWRAP_LOG
  logfd = open(CONF_MODIWRAP_LOG, O_WRONLY | O_APPEND | O_CREAT | O_NOFOLLOW, S_IRUSR | S_IWUSR);	/* errors checked later */
  if(logfd < 0)
  {
    wraperror("log open", strerror(errno));	/* doesn't use logging */
    return (-1);
  }

  if(fcntl(logfd, F_SETFD, FD_CLOEXEC))
  {
    wraperror("fcntl CLOEXEC", strerror(errno));	/* doesn't use logging */
    return (-1);
  }
#endif

  return (0);
}


void closelog(void)		/* optional */
{
#ifdef CONF_USE_SYSLOG
  closelog();
#endif

#ifdef CONF_MODIWRAP_LOG
  if(logfd >= 0)
  {
    close(logfd);
  }
#endif
}


#ifdef CONF_USE_ERRORDOCUMENT
void registererrorpage(int error, char *path)
{
  char *ptr = NULL;

  if(path != NULL)
  {
    ptr = (char *)malloc((strlen(path) + 1) * sizeof(char));

    if(ptr == NULL)
    {
      return;
    }

    strcpy(ptr, path);
  }

  switch (error)
  {
  case 401:
    if(error401 != NULL)
    {
      free(error401);
    }
    error401 = ptr;
    break;
  case 404:
    if(error404 != NULL)
    {
      free(error404);
    }
    error404 = ptr;
    break;
  case 500:
    if(error500 != NULL)
    {
      free(error500);
    }
    error500 = ptr;
    break;
  default:
    free(ptr);
    break;
  }
}
#endif


void wraperrorstatus(int status)
{
  char *statustext;

  switch (status)
  {
  case 302:
    statustext = "Moved Temporarily";
    break;
  case 401:
    statustext = "Unauthorized";
    break;
  case 404:
    statustext = "Not Found";
    break;
  case 500:
    statustext = "Internal Server Error";
    break;
  default:
    statustext = "?";
    break;
  }

  if(nphmodiwrap)
  {
    fprintf(stdout, "HTTP/1.0 %d %s\n", status, statustext);
  }
  else
  {
    fprintf(stdout, "Status: %d %s\n", status, statustext);
  }

  fflush(stdout);
}


void wrapredirect(const char *url)
{
  wraperrorstatus(302);

  fprintf(stdout,
	  "Location: %s\nContent-type: text/html\n\nMoved temporarily here: <A HREF=\"%s\">%s</A><BR>\n",
	  url, url, url);
  fflush(stdout);

#ifdef CONF_USE_SYSLOG
  syslog(LOG_INFO, "%s - %s - %s", "Redirect", url, loginfo);
#endif

#ifdef CONF_MODIWRAP_LOG
  if(logfd >= 0)
  {
    char tmp[CONF_MAXLOGINFO];

    snprintf(tmp, CONF_MAXLOGINFO, "%s %s modiwrap[%d]: %s - %s - %s\n",
	     syslogctime(), "info", getpid(), "Redirect", url, loginfo);
#ifdef CONF_SNPRINTF_NOTTERMINATED
    tmp[CONF_MAXLOGINFO - 1] = '\0';
#endif
    write(logfd, tmp, strlen(tmp));
  }
#endif
}


#ifdef CONF_USE_ERRORDOCUMENT
int wraperrordocument(int status, char *neededheaders)
{
  /*extern char **environ; */
  char *errordocument;
  char tmp[MAXSTRINGSIZE];
  int simplefile;
  FILE *fich;

  switch (status)
  {
  case 401:
    errordocument = error401;
    break;
  case 404:
    errordocument = error404;
    break;
  case 500:
    errordocument = error500;
    break;
  default:
    errordocument = NULL;
    break;
  }

  if(errordocument == NULL)	/* no error document */
  {
    return (0);
  }

  if(errordocument[0] == '"')	/* simple string */
  {
    wraperrorstatus(status);
    fprintf(stdout, "Content-type: text/plain\n");
    if(neededheaders != NULL)
    {
      fprintf(stdout, "%s", neededheaders);
    }
    fprintf(stdout, "\n%s\n", errordocument + 1);
    fflush(stdout);
    return (-1);
  }

  if(!strncmp(errordocument, "http://", 7) || !strncmp(errordocument, "https://", 8) || !strncmp(errordocument, "ftp://", 6))	/* remote redirect */
  {
    wrapredirect(errordocument);
    return (-1);
  }

  /* local redirect */
  if(strstr(errordocument, "../") == NULL)
  {
    if(errordocument[0] == '/')
    {
      strcpy(tmp, getvar("$DOCUMENT_ROOT"));
      if(strlen(tmp) < 1 || tmp[strlen(tmp) - 1] != '/')
      {
	strcat(tmp, "/");
      }
      strcat(tmp, errordocument);
    }
    else
    {
      strcpy(tmp, errordocument);
    }
  }
  else
  {
    tmp[0] = '\0';
  }

  simplefile = 1;
  if(strstr(errordocument, "cgi-bin/") != NULL
     || strstr(errordocument, ".php") != NULL
     || strstr(errordocument, "?") != NULL)
  {
    simplefile = 0;
  }
  else if(strstr(errordocument, ".html") != NULL
	  || strstr(errordocument, ".htm") != NULL)
  {
    wraperrorstatus(status);
    fprintf(stdout, "Content-type: text/html\n");
  }
  else
  {
    wraperrorstatus(status);
    fprintf(stdout, "Content-type: text/plain\n");
  }

  if(simplefile)
  {
    if(neededheaders != NULL)
    {
      fprintf(stdout, "%s\n", neededheaders);
    }
    fflush(stdout);

    fich = fopen(tmp, "r");
    if(fich != NULL)
    {
      fgets(tmp, MAXSTRINGSIZE, fich);
      while(!feof(fich))
      {
	fputs(tmp, stdout);
	fgets(tmp, MAXSTRINGSIZE, fich);
      }

      fclose(fich);
    }

    return (-1);
  }

  /* cgi or php */

  return (0);			/* not handled */

  /*redirectvars(environ);

     snprintf(tmp, 12, "%d", status);
     setvar("$REDIRECT_STATUS", tmp);

     snprintf(tmp, MAXSTRINGSIZE, "http://%s:%s%s", getvar("$SERVER_NAME"),
     getvar("$SERVER_PORT"), getvar("$REQUEST_URI"));
     setvar("$REDIRECT_URL", tmp);

     return (-1); */
}
#endif


void wrapnotfound(void)
{
#ifdef CONF_USE_ERRORDOCUMENT
  if(wraperrordocument(404, NULL))
  {
    return;
  }
#endif

  wraperrorstatus(404);

  fprintf(stdout, "Content-type: text/plain\n\nNot Found\n");
  fflush(stdout);
}


void wraperror(const char *subject, const char *detail)
{
#ifdef CONF_USE_ERRORDOCUMENT
  if(wraperrordocument(500, NULL))
  {
    return;
  }
#endif

  wraperrorstatus(500);

  fprintf(stdout,
	  "Content-type: text/plain\n\nError:\n\n%s\n%s\n\n%s\n\n",
	  subject, detail, loginfo);
  fflush(stdout);
}


void wraperrorlog(const char *subject, const char *detail)
{
#ifdef CONF_USE_SYSLOG
  syslog(LOG_ERR, "%s - %s - %s", subject, detail, loginfo);
#endif

#ifdef CONF_MODIWRAP_LOG
  if(logfd >= 0)
  {
    char tmp[CONF_MAXLOGINFO];

    snprintf(tmp, CONF_MAXLOGINFO, "%s %s modiwrap[%d]: %s - %s - %s\n",
	     syslogctime(), "ERROR", getpid(), subject, detail, loginfo);
#ifdef CONF_SNPRINTF_NOTTERMINATED
    tmp[CONF_MAXLOGINFO - 1] = '\0';
#endif
    write(logfd, tmp, strlen(tmp));
  }
#endif

  wraperror(subject, detail);
}


void wrapwarning(const char *subject, const char *detail)
{
#ifdef CONF_USE_SYSLOG
  syslog(LOG_WARNING, "%s - %s - %s", subject, detail, loginfo);
#endif

#ifdef CONF_MODIWRAP_LOG
  if(logfd >= 0)
  {
    char tmp[CONF_MAXLOGINFO];

    snprintf(tmp, CONF_MAXLOGINFO, "%s %s modiwrap[%d]: %s - %s - %s\n",
	     syslogctime(), "WARN", getpid(), subject, detail, loginfo);
#ifdef CONF_SNPRINTF_NOTTERMINATED
    tmp[CONF_MAXLOGINFO - 1] = '\0';
#endif
    write(logfd, tmp, strlen(tmp));
  }
#endif

  if(debug)
  {
    fprintf(stdout, "\nWARN: %s - %s\n\n", subject, detail);
    fflush(stdout);
  }
}


void wrapinfo(const char *subject, const char *detail)
{
#ifdef CONF_USE_SYSLOG
  syslog(LOG_INFO, "%s - %s - %s", subject, detail, loginfo);
#endif

#ifdef CONF_MODIWRAP_LOG
  if(logfd >= 0)
  {
    char tmp[CONF_MAXLOGINFO];

    snprintf(tmp, CONF_MAXLOGINFO, "%s %s modiwrap[%d]: %s - %s - %s\n",
	     syslogctime(), "info", getpid(), subject, detail, loginfo);
#ifdef CONF_SNPRINTF_NOTTERMINATED
    tmp[CONF_MAXLOGINFO - 1] = '\0';
#endif
    write(logfd, tmp, strlen(tmp));
  }
#endif

  if(debug)
  {
    fprintf(stdout, "\nINFO: %s - %s\n\n", subject, detail);
    fflush(stdout);
  }
}


void wrapaskforauth(const char *authtype, const char *authname)
{
  char tmp[MAXSTRINGSIZE];

  snprintf(tmp, MAXSTRINGSIZE, "%s %s realm=\"%s\"\n", "WWW-Authenticate:",
	   authtype, authname);
#ifdef CONF_SNPRINTF_NOTTERMINATED
  tmp[MAXSTRINGSIZE - 1] = '\0';
#endif

#ifdef CONF_USE_ERRORDOCUMENT
  if(wraperrordocument(401, tmp))
  {
    return;
  }
#endif

  wraperrorstatus(401);

  fprintf(stdout, "%s", tmp);

  fprintf(stdout, "Content-type: text/plain\n\nAuthorization Required\n");

  fflush(stdout);
}
