%option noyywrap
%{

#include"parse_config.h"
#include"parse_config.tab.h"

extern int errorcode;

int active = 0;
int sectionmatched = 0;

int config_count_column = 0;
int config_count_line = 1;
char config_count_buffer[1024];

void config_yyerror(const char *s);

#ifdef CONF_YYCLEAN
void config_yyclean(void);
#endif

void config_count(void);

%}

DIGIT 	    [0-9]
ALPHA 	    [a-zA-Z]
ALNUM 	    ({ALPHA}|{DIGIT})
WS    	    [ \t]

COMMENT     "#"[^\n]*

ID    	    ({ALPHA}|"_")({ALNUM}|"_")*
INT	    {DIGIT}+

LOCALVAR    \%{ID}
GLOBALVAR   ${ID}

SECTION     "["({ID}|"*")"]"

STRING	    \"([^\"\r\n]|(\\\\)|(\\\"))*\"

%%

break {
		if(active)
		{
		  yyterminate();
		}
	      }

{SECTION}  {
	      char *ptr;

	      config_count();

	      ptr = (char *)malloc(strlen(config_yytext) * sizeof(char));
	      if(ptr == NULL)
	      {
		config_yyerror("malloc failed");
		errorcode = -1;

		yyterminate();
	      }


	      strncpy(ptr, config_yytext + 1, strlen(config_yytext + 1) - 1);
	      #ifdef HAVE_STRNCPY_NOTTERMINATED
	      ptr[strlen(config_yytext + 1) - 1] = '\0';
	      #endif

	      if(!strcmp(ptr, "GLOBAL"))
	      {
		active = 1;
	      }
	      else if(!sectionmatched && !strcmp(ptr, "*"))
	      {
		active = 1;
	      }
	      else if(getvar("%WRAPPER_MODE") != NULL
		      && !strcmp(ptr, getvar("%WRAPPER_MODE")))
	      {
		active = 1;
		sectionmatched = 1;
	      }
	      else
	      {
		active = 0;
	      }

	      free(ptr);
      	    }

{COMMENT}   {
	      config_count();
      	    } /*eat */

{STRING}    {
	      char *ptr;

	      config_count();

	      if(active)
	      {
		ptr = extractstring(config_yytext);

		if(ptr == NULL)
		{
		  config_yyerror("malloc failed");
		  errorcode = -1;

		  yyterminate();
		}

		config_yylval.string = ptr;
		return (TOKEN_STRING);
	      }
	    }

{INT}  	    {
	      config_count();

	      if(active)
	      {
		config_yylval.number = atoi(config_yytext);
		return (TOKEN_INT);
	      }
      	    }

{GLOBALVAR} |
{LOCALVAR}  {
	      char *ptr;

	      config_count();

	      if(active)
	      {
		ptr = (char *)malloc((strlen(config_yytext) + 1) * sizeof(char));
		if(ptr == NULL)
		{
		  config_yyerror("malloc failed");
		  errorcode = -1;

		  yyterminate();
		}

		strcpy(ptr, config_yytext);

		config_yylval.string = ptr;
		return (TOKEN_VARIABLE);
	      }
	    }

home	    {
	      config_count();

	      if(active)
	      {
		return (TOKEN_HOME);
	      }
	    }

substandstat {
	      config_count();

	      if(active)
	      {
		return (TOKEN_SUBSTANDSTAT);
	      }
	    }

subst       {
	      config_count();

	      if(active)
	      {
		return (TOKEN_SUBST);
	      }
	    }

concatnotnull {
	      config_count();

	      if(active)
	      {
		return (TOKEN_CONCATNOTNULL);
	      }
	    }

concat	    {
	      config_count();

	      if(active)
	      {
		return (TOKEN_CONCAT);
	      }
	    }

"NULL"	    {
	      config_count();

	      if(active)
	      {
		return (TOKEN_NULL);
	      }
	    }

"="    	    |
"("   	    |
")"   	    |
","   	    {
	      config_count();

	      if(active)
	      {
		return (config_yylval.number = *config_yytext);
	      }
	    }

{WS}+ 	    {
	      config_count();
      	    } /* eat */

<*>\r|\n    {
	      config_count();

	      if(active)
	      {
		return (TOKEN_EOL);
	      }
	    }

<*>.  	    {
	      config_count();

	      config_yyerror("Unrecognized character");
	      errorcode = -1;

	      yyterminate();
	    }

%%

#ifdef CONF_YYCLEAN
void config_yyclean(void)
{
  yy_delete_buffer(YY_CURRENT_BUFFER);
}
#endif

void config_count(void)
{
  int i;
  static char *buf = config_count_buffer;

  for(i = 0; config_yytext[i] != '\0'; i++)
  {
    if(config_yytext[i] == '\n')
    {
      config_count_column = 0;
      config_count_line++;
      buf = config_count_buffer;
    }
    else
    {
      *buf++ = config_yytext[i];
      if(config_yytext[i] == '\t')
	config_count_column += 8 - (config_count_column & (8 - 1));
      else
	config_count_column++;
    }
    *buf = 0;
  }
}
