%option noyywrap
%{

#include"parse_htaccess.h"
#include"parse_htaccess.tab.h"

extern const char *script_filename;
extern const char *request_method;

extern int errorcode;

int filematched;
int methodmatched;

int skipcount;

int htaccess_count_column = 0;
int htaccess_count_line = 1;
char htaccess_count_buffer[1024];

void htaccess_yyerror(const char *s);

#ifdef CONF_YYCLEAN
void htaccess_yyclean(void);
#endif

void htaccess_count(void);

%}

WS    	    [ \t]

COMMENT     "#"[^\n]*

STRING	    \"([^\"\r\n]|(\\\\)|(\\\"))*\"
STRING1	    \"([^\"\r\n]|(\\\\)|(\\\"))*(\r|\n)
WORD	    [^ \t\n]+

TAGWORD     [^ >\t\n]+

FILESMATCH1 [^ >\t\n]+

%x PARAMS
%x REQUIRE
%x LIMIT
%x LIMITEXCEPT
%x FILESMATCH
%x FILES
%x FILESREGEX
%x SKIPLIMIT
%x SKIPLIMITEXCEPT
%x SKIPFILESMATCH
%x SKIPFILES
%%

<INITIAL,PARAMS>{COMMENT} {
      	      	htaccess_count();
	      } /*eat */

<PARAMS>{STRING} {
	      char *ptr;

	      htaccess_count();

	      ptr = extractstring(htaccess_yytext);

	      if(ptr == NULL)
	      {
		htaccess_yyerror("malloc failed");
		errorcode = -1;

		yyterminate();
	      }

	      htaccess_yylval.string = ptr;
	      return (TOKEN_STRING);
	    }

<PARAMS>{STRING1} {
	      char *ptr;

	      htaccess_count();

	      ptr = extractstring(htaccess_yytext);

	      if(ptr == NULL)
	      {
		htaccess_yyerror("malloc failed");
		errorcode = -1;

		yyterminate();
	      }

	      unput(yytext[strlen(yytext) - 1]);

	      htaccess_yylval.string = ptr;
	      return (TOKEN_STRING);
	    }

AuthAuthoritative {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_AUTHORITATIVE);
      	    }


AuthName {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_AUTHNAME);
	    }

AuthType {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_AUTHTYPE);
	    }

Require   {
	      htaccess_count();

	      BEGIN(REQUIRE);

	      return (TOKEN_REQUIRE);
	    }

<REQUIRE>user {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_REQUIREUSER);
      	    }

<REQUIRE>group {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_REQUIREGROUP);
      	    }

<REQUIRE>valid-user {
	      htaccess_count();

	      BEGIN(INITIAL);

	      return (TOKEN_REQUIREVALIDUSER);
      	    }

"<"LimitExcept {
	      htaccess_count();

	      BEGIN(LIMITEXCEPT);

	      methodmatched = 0;
	    }

<SKIPLIMITEXCEPT>"<"LimitExcept {
	      htaccess_count();

	      skipcount++;
	    }

"<"Limit {
	      htaccess_count();

	      BEGIN(LIMIT);

	      methodmatched = 0;
	    }

<SKIPLIMIT>"<"Limit {
	      htaccess_count();

	      skipcount++;
	    }

"<"FilesMatch {
	      htaccess_count();

	      BEGIN(FILESMATCH);

	      filematched = 0;
	    }

<SKIPFILESMATCH>"<"FilesMatch {
	      htaccess_count();

      	      skipcount++;
	    }

"<"Files {
	      htaccess_count();

	      BEGIN(FILES);

	      filematched = 0;
	    }

<SKIPFILES>"<"Files {
	      htaccess_count();

      	      skipcount++;
	    }

<LIMITEXCEPT>">" {
	      htaccess_count();

	      if(methodmatched)
	      {
		BEGIN(SKIPLIMITEXCEPT);
	      }
	      else
	      {
		BEGIN(INITIAL);
		skipcount = 1;
	      }
	    }

<LIMIT>">"  {
	      htaccess_count();

	      if(methodmatched)
	      {
		BEGIN(INITIAL);
	      }
	      else
	      {
		BEGIN(SKIPLIMIT);
		skipcount = 1;
	      }
	    }

<FILESMATCH>">"  {
	      htaccess_count();

	      if(filematched)
	      {
		BEGIN(INITIAL);
	      }
	      else
	      {
		BEGIN(SKIPFILESMATCH);
		skipcount = 1;
	      }
	    }

<FILES,FILESREGEX>">"  {
	      htaccess_count();

	      if(filematched)
	      {
		BEGIN(INITIAL);
	      }
	      else
	      {
		BEGIN(SKIPFILES);
		skipcount = 1;
	      }
	    }

<INITIAL,SKIPLIMITEXCEPT>"</"LimitExcept">" {
	      htaccess_count();

	      if(--skipcount == 0)
	      {
		BEGIN(INITIAL);
	      }
	    }

<INITIAL,SKIPLIMIT>"</"Limit">" {
	      htaccess_count();

	      if(--skipcount == 0)
	      {
		BEGIN(INITIAL);
	      }
	    }

<INITIAL,SKIPFILESMATCH>"</"FilesMatch">" {
	      htaccess_count();

	      if(--skipcount == 0)
	      {
		BEGIN(INITIAL);
	      }
	    }

<INITIAL,SKIPFILES>"</"Files">" {
	      htaccess_count();

	      if(--skipcount == 0)
	      {
		BEGIN(INITIAL);
	      }
	    }

AuthGroupFile {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_AUTHGROUPFILE);
	    }

AuthUserFile {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_AUTHUSERFILE);
	    }

ErrorDocument {
	      htaccess_count();

	      BEGIN(PARAMS);

	      return (TOKEN_ERRORDOCUMENT);
	    }

<LIMIT,LIMITEXCEPT>{TAGWORD} {
	      htaccess_count();

	      if(!strcmp(htaccess_yytext, request_method)
		 || (!strcmp(request_method, "HEAD") && !strcmp(htaccess_yytext, "GET")))
	      {
		methodmatched = 1;
	      }
	    }

<FILESMATCH,FILESREGEX>{STRING} {
	      char *ptr;

	      htaccess_count();

	      ptr = extractstring(htaccess_yytext);

	      if(ptr == NULL)
	      {
		htaccess_yyerror("malloc failed");
		errorcode = -1;

		yyterminate();
	      }

	      if(filebasematch(script_filename, ptr))
	      {
		filematched = 1;
	      }

	      free(ptr);
      	    }

<FILESMATCH,FILESREGEX>{TAGWORD} {
	      htaccess_count();

	      if(filebasematch(script_filename, htaccess_yytext))
	      {
		filematched = 1;
	      }
      	    }

<FILES>{STRING} {
	      char *ptr;

	      htaccess_count();

	      ptr = extractstring(htaccess_yytext);

	      if(ptr == NULL)
	      {
		htaccess_yyerror("malloc failed");
		errorcode = -1;

		yyterminate();
	      }


      	      if(!strcmp(ptr, "~"))
	      {
	      	BEGIN(FILESREGEX);
	      }
	      else
	      {
	      	if(filebasefnmatch(script_filename, ptr))
	      	{
		  filematched = 1;
	      	}
	      }

	      free(ptr);
      	    }

<FILES>{TAGWORD} {
	      htaccess_count();

      	      if(!strcmp(htaccess_yytext, "~"))
	      {
	      	BEGIN(FILESREGEX);
	      }
	      else
	      {
	      	if(filebasefnmatch(script_filename, htaccess_yytext))
	      	{
		  filematched = 1;
	      	}
	      }
      	    }

<PARAMS>{WORD} {
	      char *ptr;

	      htaccess_count();

	      ptr = (char *)malloc((strlen(htaccess_yytext) + 1) * sizeof(char));
	      if(ptr == NULL)
	      {
		htaccess_yyerror("malloc failed");
		errorcode = -1;

		yyterminate();
	      }


	      strcpy(ptr, htaccess_yytext);

	      htaccess_yylval.string = ptr;
	      return (TOKEN_STRING);
	    }

<*>{WS}+    {
	      htaccess_count();
      	    } /* eat */

<SKIPLIMIT,SKIPLIMITEXCEPT,SKIPFILESMATCH,SKIPFILES>.|\r|\n  {
	      htaccess_count();
      	    }

<INITIAL,PARAMS>\r|\n {
	      htaccess_count();

	      BEGIN(INITIAL);

	      return (TOKEN_EOL);
	    }

<LIMIT,LIMITEXCEPT,FILESMATCH,FILES,FILESREGEX,REQUIRE>\r|\n |
<*>.  	    {
	      htaccess_count();

	      htaccess_yyerror("Unrecognized character");
	      errorcode = -1;

	      yyterminate();
	    }

%%

#ifdef CONF_YYCLEAN
void htaccess_yyclean(void)
{
  yy_delete_buffer(YY_CURRENT_BUFFER);
}
#endif

void htaccess_count(void)
{
  int i;
  static char *buf = htaccess_count_buffer;

  for(i = 0; htaccess_yytext[i] != '\0'; i++)
  {
    if(htaccess_yytext[i] == '\n')
    {
      htaccess_count_column = 0;
      htaccess_count_line++;
      buf = htaccess_count_buffer;
    }
    else
    {
      *buf++ = htaccess_yytext[i];
      if(htaccess_yytext[i] == '\t')
	htaccess_count_column += 8 - (htaccess_count_column & (8 - 1));
      else
	htaccess_count_column++;
    }
    *buf = 0;
  }
}
