#include"rlimits.h"
#include"rlimitsp.h"

#ifdef CONF_USE_LIMITS
#ifdef CONF_USE_LIMITS_DAEMON

extern int debug;

jmp_buf gdrljump;
int gdrlfd = -1;

RETSIGTYPE getdaemonrlimits_signal(int unused)
{
  struct linger lin;

  if(gdrlfd >= 0)
  {
    lin.l_onoff = 1;
    lin.l_linger = 0;
    setsockopt(gdrlfd, SOL_TCP, SO_LINGER, &lin, sizeof(lin));	/* reset connection */

    close(gdrlfd);
  }

  longjmp(gdrljump, -1);
}

int getdaemonrlimits_1(const char *name, const uid_t uid, const gid_t gid)
{
#ifdef CONF_USE_GETHOSTBYNAME
  struct hostent *he;
#endif
  struct sockaddr_in sin;
  char sockbuf[CONF_SOCKBUFSIZE + 1];
  char *start;
  char *equal;
  int i, n;

  sin.sin_family = AF_INET;
  if(isipaddr(CONF_LIMITS_DAEMON_ADDR))
  {
    inet_aton(CONF_LIMITS_DAEMON_ADDR, &sin.sin_addr);
  }
  else
  {
#ifdef CONF_USE_GETHOSTBYNAME
    he = gethostbyname(CONF_LIMITS_DAEMON_ADDR);
    if(he == NULL)
    {
      wrapwarning("getdaemonrlimits gethostbyname", hstrerror(h_errno));
      return (-1);
    }
    memcpy(&sin.sin_addr, he->h_addr, he->h_length);
#else
    wrapwarning("gethostbyname failed",
		"support isn't compiled and daemon addr isn't an IP");
    return (-1);
#endif
  }
  sin.sin_port = htons(CONF_LIMITS_DAEMON_PORT);

  gdrlfd = socket(AF_INET, SOCK_STREAM, PF_UNSPEC);
  if(gdrlfd < 0)
  {
    wrapwarning("getdaemonrlimits socket", strerror(errno));
    return (-1);
  }

  i = -1;
  setsockopt(gdrlfd, SOL_TCP, SO_KEEPALIVE, &i, sizeof(i));

  if(connect(gdrlfd, (struct sockaddr *)&sin, sizeof(sin)))
  {
    wrapwarning("getdaemonrlimits connect", strerror(errno));
    close(gdrlfd);
    return (-1);
  }

  snprintf(sockbuf, CONF_SOCKBUFSIZE, "%d %d %s\015\012", uid, gid, name);
#ifdef HAVE_SNPRINTF_NOTTERMINATED
  sockbuf[CONF_SOCKBUFSIZE - 1] = '\0';
#endif
  if(send(gdrlfd, sockbuf, strlen(sockbuf), 0) < 0)
  {
    wrapwarning("getdaemonrlimits send", strerror(errno));
    close(gdrlfd);
    return (-1);
  }

  start = sockbuf;
  n = 0;
  while(n < CONF_SOCKBUFSIZE
	&& (i =
	    recv(gdrlfd, sockbuf + n, CONF_SOCKBUFSIZE - n,
		 MSG_NOSIGNAL)) > 0)
  {
    n += i;
  }
  sockbuf[n] = '\0';

  if(debug)
  {
    fprintf(stdout, "daemon limits: %s\n", sockbuf);
    fflush(stdout);
  }

  if(strstr(sockbuf, "\015\012") == NULL)
  {
    wrapwarning("getdaemonrlimits recv", "Incomplete answer");
    close(gdrlfd);
    return (-1);
  }

  start = sockbuf;
  while(*start != '\0' && isspace(*start))
  {
    start++;
  }
  do
  {
    equal = strchr(start, '=');
    if(equal == NULL)
    {
      break;
    }
    *equal = '\0';
    equal++;

    i = 0;
    while(mymw_rlimits[i].mw_desc != NULL)
    {
      if(!strcmp(start, mymw_rlimits[i].mw_desc))
      {
	mymw_rlimits[i].mw_val = (rlim_t) strtoul(equal, (char **)NULL, 10);
	break;
      }
      i++;
    }

    start = strchr(equal, ' ');
    if(start == NULL)
    {
      start = strchr(equal, '\t');
    }
    if(start != NULL)
    {
      while(*start != '\0' && isspace(*start))
      {
	start++;
      }
    }
  }
  while(start != NULL);

  close(gdrlfd);
  return (0);
}

int getdaemonrlimits(const char *name, const uid_t uid, const gid_t gid)
{
#ifdef CONF_LIMITS_DAEMON_TIMEOUT
  RETSIGTYPE(*oldalarm) (int);
#endif
  int retval;

#ifdef CONF_LIMITS_DAEMON_TIMEOUT
  oldalarm = signal(SIGALRM, getdaemonrlimits_signal);
  if(oldalarm == SIG_ERR)
  {
    wraperrorlog("getdaemonrlimits signal failed", strerror(errno));
    return (-1);
  }

  alarm(CONF_LIMITS_DAEMON_TIMEOUT);

  if(setjmp(gdrljump) == 0)
  {
#endif
    retval = getdaemonrlimits_1(name, uid, gid);
#ifdef CONF_LIMITS_DAEMON_TIMEOUT
  }
  else
  {
    signal(SIGALRM, oldalarm);
    wrapwarning("getdaemonrlimits", "timeout");
    return (-2);
  }

  alarm(0);
  signal(SIGALRM, oldalarm);
#endif

  return (retval);
}

#endif

int setrlimitsandalarm(const char *name, const uid_t uid, const gid_t gid,
		       void (*timealrm) (int))
{
  struct rlimit myrlimit;
  int i;

#ifdef CONF_USE_LIMITS_DAEMON
  getdaemonrlimits(name, uid, gid);	/* errors are non-fatal */
#endif

  for(i = 0; mymw_rlimits[i].mw_desc != NULL; i++)
  {
    if(debug)
    {
      fprintf(stdout, "%s=%ld\n", mymw_rlimits[i].mw_desc,
	      mymw_rlimits[i].mw_val);
      fflush(stdout);
    }

    if(mymw_rlimits[i].mw_limit != -1)	/* normal rlimits */
    {
      if(mymw_rlimits[i].mw_val != (rlim_t) (-1))	/* infinite */
      {
	myrlimit.rlim_cur = mymw_rlimits[i].mw_val;
	myrlimit.rlim_max = mymw_rlimits[i].mw_val;
      }
      else
      {
	myrlimit.rlim_cur = RLIM_INFINITY;
	myrlimit.rlim_max = RLIM_INFINITY;
      }

      if(setrlimit(mymw_rlimits[i].mw_limit, &myrlimit))
      {
	wraperrorlog("setrlimit failed", strerror(errno));
	return (-1);
      }
    }
    else
    {
#ifdef CONF_NICE
      if(!strcmp(mymw_rlimits[i].mw_desc, "NICE"))
      {
	if(mymw_rlimits[i].mw_val != (rlim_t) (-1))
	{
#ifdef HAVE_SETPRIORITY
	  setpriority(PRIO_PROCESS, getpid(), (int)mymw_rlimits[i].mw_val);
#else
#ifdef HAVE_NICE
	  nice((int)mymw_rlimits[i].mw_val);
#else
#warning NICE requested but not available
#endif
#endif
	}
	else
	{
#ifdef HAVE_SETPRIORITY
	  setpriority(PRIO_PROCESS, getpid(), 0);
#else
#ifdef HAVE_NICE
	  nice(0);		/* oops! relative! */
#else
#warning NICE requested but not available
#endif
#endif
	}
      }
#endif
#ifdef CONF_TIMEALRM
      if(!strcmp(mymw_rlimits[i].mw_desc, "TIME"))
      {
	if(signal(SIGALRM, timealrm) == SIG_ERR)
	{
	  wraperrorlog("timealrm signal failed", strerror(errno));
	  return (-1);
	}
	if(mymw_rlimits[i].mw_val != (rlim_t) (-1))
	{
	  alarm((unsigned int)mymw_rlimits[i].mw_val);
	}
	else
	{
	  alarm(0);
	}
      }
#endif
    }
  }

  if(debug)
  {
    fprintf(stdout, "\n");
    fflush(stdout);
  }
  return (0);
}

#endif
