#ifndef _RLIMITSP_H
#define _RLIMITSP_H

#ifdef HAVE_CONFIG_H
#include"config.h"
#endif

#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_SYS_RESOURCE_H
#include<sys/resource.h>
#endif

#ifdef HAVE_UNISTD_H
#include<unistd.h>
#endif

#ifdef CONF_USE_LIMITS_DAEMON
#ifdef HAVE_SYS_SOCKET_H
#include<sys/socket.h>
#endif

#ifdef HAVE_NETINET_IN_H
#include<netinet/in.h>
#endif

#ifdef HAVE_NETINET_TCP_H
#include<netinet/tcp.h>
#endif

#ifdef HAVE_ARPA_INET_H
#include<arpa/inet.h>
#endif


#ifdef HAVE_SYS_TYPES_H
#include<sys/types.h>
#endif

#ifdef HAVE_SETJMP_H
#include<setjmp.h>
#endif

#ifdef HAVE_SIGNAL_H
#include<signal.h>
#endif

#ifdef HAVE_STDLIB_H
#include<stdlib.h>
#endif

#ifdef HAVE_CTYPE_H
#include<ctype.h>
#endif

#ifdef CONF_USE_GETHOSTBYNAME
#ifdef HAVE_NETDB_H
#include<netdb.h>
#endif
#endif

int isipaddr(const char *str)
{
  int i;

  if(str == NULL)
  {
    return (0);
  }
  for(i = 0; i < strlen(str); i++)
  {
    if(!isdigit(str[i]) && str[i] != '.')
    {
      return (0);
    }
  }
  return (-1);
}

#endif

/*
  Modiwrap Daemon <-> Client dialog:
  
  Client:
  UID GID USERNAME\015\012
  
  ex:
  0 0 foo\015\012
  
  Server:
  modiwrap_rlimits[i].dw_desc=modiwrap_rlimits[i].dw_val
  SPACE modiwrap_rlimits[j].dw_desc=modiwrap_rlimits[j].dw_val
  SPACE ...
  CRLF
  
  ex:
  NICE=0 TIME=60 CPU=10 VMEM=2500000 FSIZE=2500000 NFILE=32 NPROC=32\015\012

  unknown desc -> ignored
  not set value -> default value
  -1 -> unlimited
  values set from left to right, so last value counts
  
  see available keywords below, man setrlimit to know exact function on your system
*/

#if defined(CONF_USE_LIMITS)
struct modiwrap_rlimits
{
  int mw_limit;
  char *mw_desc;
  rlim_t mw_val;
};

#ifndef MODIWRAPD

struct modiwrap_rlimits mymw_rlimits[] = {
#if defined(CONF_NICE)
  {-1, "NICE", CONF_NICE},
#endif
#if defined(CONF_TIMEALRM)
  {-1, "TIME", CONF_TIMEALRM},
#endif
#if defined(RLIMIT_CPU) && defined(CONF_RLIMIT_CPU)
  {RLIMIT_CPU, "CPU", CONF_RLIMIT_CPU},
#else
#if !defined(RLIMIT_CPU) && defined(CONF_RLIMIT_CPU)
#warning RLIMIT_CPU requested but not available
#endif
#endif
#if defined(RLIMIT_VMEM) && defined(CONF_RLIMIT_VMEM)
  {RLIMIT_VMEM, "VMEM", CONF_RLIMIT_VMEM},
#else
#if !defined(RLIMIT_VMEM) && defined(CONF_RLIMIT_VMEM)
#warning RLIMIT_VMEM requested but not available
#endif
#endif
#if defined(RLIMIT_FSIZE) && defined(CONF_RLIMIT_FSIZE)
  {RLIMIT_FSIZE, "FSIZE", CONF_RLIMIT_FSIZE},
#else
#if !defined(RLIMIT_FSIZE) && defined(CONF_RLIMIT_FSIZE)
#warning RLIMIT_FSIZE requested but not available
#endif
#endif
#if defined(RLIMIT_DATA) && defined(CONF_RLIMIT_DATA)
  {RLIMIT_DATA, "DATA", CONF_RLIMIT_DATA},
#else
#if !defined(RLIMIT_DATA) && defined(CONF_RLIMIT_DATA)
#warning RLIMIT_DATA requested but not available
#endif
#endif
#if defined(RLIMIT_STACK) && defined(CONF_RLIMIT_STACK)
  {RLIMIT_STACK, "STACK", CONF_RLIMIT_STACK},
#else
#if !defined(RLIMIT_STACK) && defined(CONF_RLIMIT_STACK)
#warning RLIMIT_STACK requested but not available
#endif
#endif
#if defined(RLIMIT_AS) && defined(CONF_RLIMIT_AS)
  {RLIMIT_AS, "AS", CONF_RLIMIT_AS},
#else
#if !defined(RLIMIT_AS) && defined(CONF_RLIMIT_AS)
#warning RLIMIT_AS requested but not available
#endif
#endif
#if defined(RLIMIT_CORE) && defined(CONF_RLIMIT_CORE)
  {RLIMIT_CORE, "CORE", CONF_RLIMIT_CORE},
#else
#if !defined(RLIMIT_CORE) && defined(CONF_RLIMIT_CORE)
#warning RLIMIT_CORE requested but not available
#endif
#endif
#if defined(RLIMIT_RSS) && defined(CONF_RLIMIT_RSS)
  {RLIMIT_RSS, "RSS", CONF_RLIMIT_RSS},
#else
#if !defined(RLIMIT_RSS) && defined(CONF_RLIMIT_RSS)
#warning RLIMIT_RSS requested but not available
#endif
#endif
#if defined(RLIMIT_NPROC) && defined(CONF_RLIMIT_NPROC)
  {RLIMIT_NPROC, "NPROC", CONF_RLIMIT_NPROC},
#else
#if !defined(RLIMIT_NPROC) && defined(CONF_RLIMIT_NPROC)
#warning RLIMIT_NPROC requested but not available
#endif
#endif
#if defined(RLIMIT_NOFILE) && defined(CONF_RLIMIT_NOFILE)
  {RLIMIT_NOFILE, "NOFILE", CONF_RLIMIT_NOFILE},
#else
#if !defined(RLIMIT_NOFILE) && defined(CONF_RLIMIT_NOFILE)
#warning RLIMIT_NOFILE requested but not available
#endif
#endif
#if defined(RLIMIT_MEMLOCK) && defined(CONF_RLIMIT_MEMLOCK)
  {RLIMIT_MEMLOCK, "MEMLOCK", CONF_RLIMIT_MEMLOCK},
#else
#if !defined(RLIMIT_MEMLOCK) && defined(CONF_RLIMIT_MEMLOCK)
#warning RLIMIT_MEMLOCK requested but not available
#endif
#endif
#if defined(RLIMIT_LOCKS) && defined(CONF_RLIMIT_LOCKS)
  {RLIMIT_LOCKS, "LOCKS", CONF_RLIMIT_LOCKS},
#else
#if !defined(RLIMIT_LOCKS) && defined(CONF_RLIMIT_LOCKS)
#warning RLIMIT_LOCKS requested but not available
#endif
#endif
  {0, NULL, 0}
};
#endif

#endif

#endif
