#include"subst.h"

/* used for debug only */
#define REGERRBUFLEN 1024
#define DESTSTRINGSIZE 1024


int subst(const char *argstring, const char *argregexp, const char *argsubst,
	  char *deststring, size_t deststringsize)
{
  regex_t myreg;
  regmatch_t mymatch[CONF_MAXREGMATCH];

#ifdef DEBUG
  char regerrbuf[REGERRBUFLEN];
#endif
  const char *ptrold;
  const char *ptr;
  int i, j;

  if(deststring == NULL || deststringsize < 1)
  {
    return (-2);
  }

  deststring[0] = '\0';

  if(argregexp == NULL || argstring == NULL)
  {
    return (-2);
  }

  i = regcomp(&myreg, argregexp, REG_EXTENDED);
  if(i != 0)
  {
#ifdef DEBUG
    regerror(i, &myreg, regerrbuf, REGERRBUFLEN);
    fprintf(stderr, "%s\n", regerrbuf);
    regfree(&myreg);
#endif
    return (-1);
  }

  i = regexec(&myreg, argstring, CONF_MAXREGMATCH, mymatch, 0);
  if(i != 0)
  {
#ifdef DEBUG
    regerror(i, &myreg, regerrbuf, REGERRBUFLEN);
    fprintf(stderr, "%s\n", regerrbuf);
    regfree(&myreg);
#endif
    return (-1);
  }

  regfree(&myreg);

#ifdef DEBUG
  fprintf(stderr, "%s\n", argstring);
  for(i = 0; i < CONF_MAXREGMATCH; i++)
  {
    if(mymatch[i].rm_so == -1)
    {
      continue;
    }
    for(j = 0; j < mymatch[i].rm_so; j++)
    {
      fprintf(stderr, " ");
    }
    for(; j < mymatch[i].rm_eo; j++)
    {
      fprintf(stderr, "-");
    }
    for(; j < strlen(argstring); j++)
    {
      fprintf(stderr, " ");
    }
    fprintf(stderr, " %d, %d\n", mymatch[i].rm_so, mymatch[i].rm_eo);
  }
#endif

  if(argsubst == NULL || deststring == NULL)
  {
    return (-2);
  }

  j = 0;
  ptrold = argsubst;
  ptr = strchr(ptrold, '\\');
  while(ptr != NULL && j < deststringsize - 1)
  {
    i = ptr - ptrold;
    if(j + i < deststringsize - 1)
    {
      if(i > 0)
      {
	strncpy(deststring + j, ptrold, i);
	j += i;
      }
    }
    else
    {
      strncpy(deststring + j, ptrold, deststringsize - j - 1);
      j = deststringsize - 1;
    }

    ptrold = ptr;
    ptr++;
    if(isdigit(*ptr))		/* found var */
    {
      i = *ptr - '0';
      ptr++;
      ptrold = ptr;

      if(i < CONF_MAXREGMATCH && mymatch[i].rm_so != -1
	 && mymatch[i].rm_eo - mymatch[i].rm_so > 0)
      {
	if(j + mymatch[i].rm_eo - mymatch[i].rm_so < deststringsize - 1)
	{
	  strncpy(deststring + j, argstring + mymatch[i].rm_so,
		  mymatch[i].rm_eo - mymatch[i].rm_so);
	  j += mymatch[i].rm_eo - mymatch[i].rm_so;
	}
	else
	{
	  strncpy(deststring + j, argstring + mymatch[i].rm_so,
		  deststringsize - j - 1);
	  j = deststringsize - 1;
	}
      }
    }
    else
    {
      if(j + 1 < deststringsize - 1
	 && ptrold + 1 < argsubst + strlen(argsubst))
      {
	strncpy(deststring + j, ptrold + 1, 1);	/* '\\?' */
	j++;
	ptrold += 2;
      }
      else
      {
	strncpy(deststring + j, ptrold, 1);	/* '\\' */
	j++;
	ptrold++;
      }
    }

    ptr = strchr(ptrold, '\\');
  }

  /* end */
  i = argsubst + strlen(argsubst) - ptrold;
  if(j + i < deststringsize - 1)
  {
    if(i > 0)
    {
      strncpy(deststring + j, ptrold, i);
      j += i;
    }
  }
  else
  {
    strncpy(deststring + j, ptrold, deststringsize - j - 1);
    j = deststringsize - 1;
  }

  deststring[j] = '\0';

  return (0);
}

int substandstat(const char *argstring, const char *argregexp,
		 const char *argsubst, char *deststring,
		 size_t deststringsize)
{
  int depth = CONF_MIN_DEPTH;
  char tmpsubst[DESTSTRINGSIZE];
  char tmpregexp[DESTSTRINGSIZE];
  char tmpdeststring[DESTSTRINGSIZE];
  struct stat sbuf;

  if(deststring == NULL || deststringsize < 1)
  {
    return (-2);
  }

  deststring[0] = '\0';

  do
  {
    snprintf(tmpsubst, DESTSTRINGSIZE, "\\1%d\\2", depth);
#ifdef HAVE_SNPRINTF_NOTTERMINATED
    tmpsubst[DESTSTRINGSIZE - 1] = '\0';
#endif
    tmpregexp[0] = '\0';
    subst(argregexp, "^([^\\]*)\\\\D(.*)$", tmpsubst, tmpregexp,
	  DESTSTRINGSIZE);

#ifdef DEBUG
    fprintf(stderr, "%s\n", tmpregexp);
#endif

    tmpdeststring[0] = '\0';
    subst(argstring, tmpregexp, argsubst, tmpdeststring, DESTSTRINGSIZE);
    if(tmpdeststring[0] == '\0')	/* No match */
    {
      break;
    }
    if(lstat(tmpdeststring, &sbuf) == 0)	/* exists */
    {
      strncpy(deststring, tmpdeststring, deststringsize - 1);
#ifdef HAVE_STRNCPY_NOTTERMINATED
      deststring[deststringsize - 1] = '\0';
#endif
    }
    else
    {
      break;
    }

    depth++;
  }
  while(depth < CONF_MAX_DEPTH);

  if(depth >= CONF_MAX_DEPTH)
  {
    deststring[0] = '\0';
    return (-1);
  }

  if(deststring[0] == '\0')	/* no exists-match found */
  {
    return (-1);
  }

  return (0);
}


int filebasematch(const char *filename, const char *regexp)
{
  const char *ptr;
  char *basename;
  regex_t myreg;
  regmatch_t mymatch[CONF_MAXREGMATCH];

  ptr = strrchr(filename, '/');
  if(ptr == NULL)		/* only filename */
  {
    basename = (char *)malloc((strlen(filename) + 1) * sizeof(char));

    if(basename == NULL)
    {
      return (0);
    }

    strcpy(basename, filename);
  }
  else
  {
    basename = (char *)malloc((strlen(ptr + 1) + 1) * sizeof(char));

    if(basename == NULL)
    {
      return (0);
    }

    strcpy(basename, ptr + 1);
  }

  if(regcomp(&myreg, regexp, REG_EXTENDED))	/* bad regex */
  {
    free(basename);
    return (0);
  }

  if(regexec(&myreg, basename, CONF_MAXREGMATCH, mymatch, 0))	/* nomatch */
  {
    regfree(&myreg);
    free(basename);
    return (0);
  }

  regfree(&myreg);
  free(basename);
  return (-1);
}

int filebasefnmatch(const char *filename, const char *regexp)
{
  const char *ptr;
  char *basename;

  ptr = strrchr(filename, '/');
  if(ptr == NULL)		/* only filename */
  {
    basename = (char *)malloc((strlen(filename) + 1) * sizeof(char));

    if(basename == NULL)
    {
      return (0);
    }

    strcpy(basename, filename);
  }
  else
  {
    basename = (char *)malloc((strlen(ptr + 1) + 1) * sizeof(char));

    if(basename == NULL)
    {
      return (0);
    }

    strcpy(basename, ptr + 1);
  }

#ifdef HAVE_FNMATCH
  if(fnmatch(regexp, basename, 0) != 0)
  {
    free(basename);
    return (0);
  }
#else
#error no workaround for fnmatch yet
#endif

  free(basename);
  return (-1);
}

#ifdef TEST
int main(int argc, char **argv)
{
  char *argregexp = NULL;
  char *argsubst = NULL;
  char *argstring = NULL;
  char deststring[DESTSTRINGSIZE];
  int dostat = 0;

  if(argc <= 1)
  {
    fprintf(stderr, "Usage: %s [-stat] <string> <regexp> [<subst>]\n",
	    argv[0]);
    return (-1);
  }
  if(argc == 3)
  {
    argstring = argv[1];
    argregexp = argv[2];
  }
  else if(argc == 4)
  {
    argstring = argv[1];
    argregexp = argv[2];
    argsubst = argv[3];
  }
  else if(argc == 5 && !strcmp(argv[1], "-stat"))
  {
    dostat = 1;
    argstring = argv[2];
    argregexp = argv[3];
    argsubst = argv[4];
  }
  else
  {
    fprintf(stderr, "arg error\n");
    return (-1);
  }

  /*deststring[0] = '\0'; */
  if(!dostat)
  {
    if(subst(argstring, argregexp, argsubst, deststring, DESTSTRINGSIZE))
    {
      fprintf(stderr, "error returned\n");
    }
  }
  else
  {
    if(substandstat
       (argstring, argregexp, argsubst, deststring, DESTSTRINGSIZE))
    {
      fprintf(stderr, "error returned\n");
    }
  }

  fprintf(stdout, "%s\n", deststring);

  return (0);
}
#endif
