#include"var.h"

int localvarsinitialized = 0;

#ifdef CONF_PARANOID_ENV
char *paranoid_env[] = {
  "AUTH_TYPE",
  "CONTENT_LENGTH",
  "CONTENT_TYPE",
  "DATE_GMT",
  "DATE_LOCAL",
  "DOCUMENT_NAME",
  "DOCUMENT_PATH_INFO",
  "DOCUMENT_ROOT",
  "DOCUMENT_URI",
  "FILEPATH_INFO",
  "GATEWAY_INTERFACE",
  "LAST_MODIFIED",
  "PATH",
  "PATH_INFO",
  "PATH_TRANSLATED",
  "QUERY_STRING",
  "QUERY_STRING_UNESCAPED",
  "REMOTE_ADDR",
  "REMOTE_HOST",
  "REMOTE_IDENT",
  "REMOTE_PORT",
  "REMOTE_USER",
  "REDIRECT_QUERY_STRING",
  "REDIRECT_STATUS",
  "REDIRECT_URL",
  "REQUEST_METHOD",
  "REQUEST_URI",
  "SCRIPT_FILENAME",
  "SCRIPT_NAME",
  "SCRIPT_URI",
  "SCRIPT_URL",
  "SERVER_ADMIN",
  "SERVER_NAME",
  "SERVER_ADDR",
  "SERVER_PORT",
  "SERVER_PROTOCOL",
  "SERVER_SOFTWARE",
  "UNIQUE_ID",
  "USER_NAME",
  "TZ",
  NULL
};
#endif

struct localvars
{
  char *lv_name;
  char *lv_val;
};

struct localvars mylocalvars[CONF_MAXLOCALVARS];

int setglobalvar(const char *name, const char *value)
{
  if(name == NULL)
  {
    return (-1);
  }

  if(value == NULL)
  {
    unsetenv(name);
    return (0);
  }

  return (setenv(name, value, 1));
}

int unsetglobalvar(const char *name)
{
  return (setglobalvar(name, NULL));
}

char *getglobalvar(const char *name)
{
  char *ptr;

  if(name == NULL)
  {
    return (NULL);
  }

  ptr = getenv(name);

  /*if (ptr != NULL)
     {
     ptr = strchr(ptr, '=');
     } */

  return (ptr);
}

void initlocalvars()
{
  int i;

  for(i = 0; i < CONF_MAXLOCALVARS; i++)
  {
    mylocalvars[i].lv_name = NULL;
    mylocalvars[i].lv_val = NULL;
  }

  localvarsinitialized = 1;
}

int setlocalvar(const char *name, const char *value)
{
  int slot = -1;
  int i;

  if(!localvarsinitialized)
  {
    initlocalvars();
  }

  if(name == NULL)
  {
    return (-1);
  }

  for(i = 0; i < CONF_MAXLOCALVARS; i++)
  {
    if(mylocalvars[i].lv_name != NULL)
    {
      if(!strcmp(mylocalvars[i].lv_name, name))
      {
	slot = i;
	break;
      }
    }
    else
    {
      if(slot == -1)
      {
	slot = i;
      }
    }
  }

  if(slot < 0 || slot >= CONF_MAXLOCALVARS)
  {
    return (-1);
  }

  if(mylocalvars[slot].lv_name != NULL && value == NULL)
  {
    free(mylocalvars[slot].lv_name);
    mylocalvars[slot].lv_name = NULL;
  }
  else if(mylocalvars[slot].lv_name == NULL && value != NULL)
  {
    mylocalvars[slot].lv_name =
      (char *)malloc((strlen(name) + 1) * sizeof(char));

    if(mylocalvars[slot].lv_name == NULL)
    {
      wraperrorlog("new local var name", "malloc failed");
      return (-1);
    }

    strcpy(mylocalvars[slot].lv_name, name);
  }

  if(mylocalvars[slot].lv_val != NULL)
  {
    free(mylocalvars[slot].lv_val);
  }
  if(value != NULL)
  {
    mylocalvars[slot].lv_val =
      (char *)malloc((strlen(value) + 1) * sizeof(char));

    if(mylocalvars[slot].lv_val == NULL)
    {
      if(mylocalvars[slot].lv_name != NULL)
      {
	free(mylocalvars[slot].lv_name);
	mylocalvars[slot].lv_name = NULL;
      }
      wraperrorlog("new local var value", "malloc failed");
      return (-1);
    }

    strcpy(mylocalvars[slot].lv_val, value);
  }
  else
  {
    mylocalvars[slot].lv_val = NULL;
  }

  return (0);
}

int unsetlocalvar(const char *name)
{
  return (setlocalvar(name, NULL));
}

char *getlocalvar(const char *name)
{
  int i;

  if(!localvarsinitialized)
  {
    initlocalvars();
  }

  if(name == NULL)
  {
    return (NULL);
  }

  for(i = 0; i < CONF_MAXLOCALVARS; i++)
  {
    if(mylocalvars[i].lv_name != NULL)
    {
      if(!strcmp(mylocalvars[i].lv_name, name))
      {
	return (mylocalvars[i].lv_val);
      }
    }
  }

  return (NULL);
}

int setvar(const char *name, const char *value)
{
  if(name == NULL)
  {
    return (-1);
  }

  if(name[0] == '%')
  {
    return (setlocalvar(name + 1, value));
  }

  if(name[0] == '$')
  {
    return (setglobalvar(name + 1, value));
  }

  return (-1);
}

int unsetvar(const char *name)
{
  if(name == NULL)
  {
    return (-1);
  }

  if(name[0] == '%')
  {
    return (unsetlocalvar(name + 1));
  }

  if(name[0] == '$')
  {
    return (unsetglobalvar(name + 1));
  }

  return (-1);
}

char *getvar(const char *name)
{
  if(name == NULL)
  {
    return (NULL);
  }

  if(name[0] == '%')
  {
    return (getlocalvar(name + 1));
  }

  if(name[0] == '$')
  {
    return (getglobalvar(name + 1));
  }

  return (NULL);
}

void printvars(char **env)
{
  int i;

  for(i = 0; i < CONF_MAXLOCALVARS; i++)
  {
    if(mylocalvars[i].lv_name != NULL)
    {
      if(mylocalvars[i].lv_val != NULL)
      {
	fprintf(stdout, "%%%s=%s\n", mylocalvars[i].lv_name,
		mylocalvars[i].lv_val);
      }
      else
      {
	fprintf(stdout, "%%%s is NULL\n", mylocalvars[i].lv_name);
      }
    }
  }

  i = 0;
  while(env[i] != NULL)
  {
    fprintf(stdout, "$%s\n", env[i]);
    i++;
  }

  fprintf(stdout, "\n");
}

void cleanlocalvars(void)
{
  int i;

  for(i = 0; i < CONF_MAXLOCALVARS; i++)
  {
    if(mylocalvars[i].lv_name != NULL)
    {
      free(mylocalvars[i].lv_name);
      mylocalvars[i].lv_name = NULL;
      if(mylocalvars[i].lv_val != NULL)
      {
	free(mylocalvars[i].lv_val);
	mylocalvars[i].lv_val = NULL;
      }
    }
  }
}

#ifdef CONF_PARANOID_ENV
char **createsaveenv(char **env)
{
  char **newenv;
  int i;
  int j;
  int k;

  i = 0;
  while(env[i] != NULL)
  {
    i++;
  }

  newenv = (char **)malloc((sizeof(char *) * (i + 1)));
  if(newenv == NULL)
  {
    wraperrorlog("create safe env", "malloc failed");
    return (NULL);
  }
  newenv[0] = NULL;

  k = 0;
  i = 0;
  while(env[i] != NULL)
  {
    if(!strncmp(env[i], "HTTP_", 5))	/* auth has been deleted already */
    {
      newenv[k] = env[i];
      k++;
    }
    else if(!strncmp(env[i], "REDIRECT_", 9))
    {
      newenv[k] = env[i];
      k++;
    }
    else
    {
      j = 0;
      while(paranoid_env[j] != NULL)
      {
	if(!strncmp(env[i], paranoid_env[j], strlen(paranoid_env[j]))
	   && env[i][strlen(paranoid_env[j])] == '=')
	  /* last char should be at worst '\0' */
	{
	  newenv[k] = env[i];
	  k++;
	  break;
	}

	j++;
      }
    }

    i++;
  }

  newenv[k] = NULL;

  return (newenv);
}
#endif

#ifdef CONF_USE_ERRORDOCUMENT
/*void redirectvars(char **env)
{
  int i;
  char var[MAXSTRINGSIZE];
  char tmp[MAXSTRINGSIZE];
  char *ptr;

  i = 0;
  while (env[i] != NULL)
  {
    strcpy(var, env[i]);
    ptr = strchr(var, '=');
    if (ptr != NULL)
    {
      *ptr = '\0';
    }

    if (!strncmp(var, "REDIRECT_", 9))
    {
      unsetenv(var);
    }

    i++;
  }

  i = 0;
  while (env[i] != NULL)
  {
    strcpy(var, env[i]);
    ptr = strchr(var, '=');
    if (ptr != NULL)
    {
      *ptr = '\0';
    }

    if (!strncmp(var, "PATH_", 5) || !strncmp(var, "QUERY_", 6)
	|| !strncmp(var, "REQUEST_", 8) || !strncmp(var, "SCRIPT_", 7))
    {
      snprintf(tmp, MAXSTRINGSIZE, "%s%s", "REDIRECT_", env[i]);

      unsetvar(var);
      putenv(tmp);
    }
    else
    {
      unsetenv(var);
    }

    i++;
  }
}*/
#endif
